from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.args import feature_view__client_pb2 as _feature_view__client_pb2
from tecton_proto.args import pipeline__client_pb2 as _pipeline__client_pb2
from tecton_proto.common import aggregation_function__client_pb2 as _aggregation_function__client_pb2
from tecton_proto.common import compute_mode__client_pb2 as _compute_mode__client_pb2
from tecton_proto.common import data_source_type__client_pb2 as _data_source_type__client_pb2
from tecton_proto.common import framework_version__client_pb2 as _framework_version__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from tecton_proto.common import time_window__client_pb2 as _time_window__client_pb2
from tecton_proto.data import fco_metadata__client_pb2 as _fco_metadata__client_pb2
from tecton_proto.data import fv_materialization__client_pb2 as _fv_materialization__client_pb2
from tecton_proto.data import realtime_compute__client_pb2 as _realtime_compute__client_pb2
from tecton_proto.modelartifactservice import model_artifact_service__client_pb2 as _model_artifact_service__client_pb2
from tecton_proto.validation import validator__client_pb2 as _validator__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DELTA_OFFLINE_STORE_VERSION_1: DeltaOfflineStoreVersion
DELTA_OFFLINE_STORE_VERSION_2: DeltaOfflineStoreVersion
DELTA_OFFLINE_STORE_VERSION_UNSPECIFIED: DeltaOfflineStoreVersion
DESCRIPTOR: _descriptor.FileDescriptor
MATERIALIZATION_TIME_RANGE_POLICY_FAIL_IF_OUT_OF_RANGE: MaterializationTimeRangePolicy
MATERIALIZATION_TIME_RANGE_POLICY_FILTER_TO_RANGE: MaterializationTimeRangePolicy
MATERIALIZATION_TIME_RANGE_POLICY_UNSPECIFIED: MaterializationTimeRangePolicy
PARQUET_OFFLINE_STORE_VERSION_1: ParquetOfflineStoreVersion
PARQUET_OFFLINE_STORE_VERSION_2: ParquetOfflineStoreVersion
PARQUET_OFFLINE_STORE_VERSION_UNSPECIFIED: ParquetOfflineStoreVersion

class Aggregate(_message.Message):
    __slots__ = ["batch_sawtooth_tile_size", "description", "function", "function_params", "input_feature_name", "output_feature_name", "tags", "time_window", "window"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    BATCH_SAWTOOTH_TILE_SIZE_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    FUNCTION_FIELD_NUMBER: ClassVar[int]
    FUNCTION_PARAMS_FIELD_NUMBER: ClassVar[int]
    INPUT_FEATURE_NAME_FIELD_NUMBER: ClassVar[int]
    OUTPUT_FEATURE_NAME_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    TIME_WINDOW_FIELD_NUMBER: ClassVar[int]
    WINDOW_FIELD_NUMBER: ClassVar[int]
    batch_sawtooth_tile_size: _duration_pb2.Duration
    description: str
    function: _aggregation_function__client_pb2.AggregationFunction
    function_params: _aggregation_function__client_pb2.AggregationFunctionParams
    input_feature_name: str
    output_feature_name: str
    tags: _containers.ScalarMap[str, str]
    time_window: _time_window__client_pb2.TimeWindow
    window: _duration_pb2.Duration
    def __init__(self, input_feature_name: Optional[str] = ..., output_feature_name: Optional[str] = ..., function: Optional[Union[_aggregation_function__client_pb2.AggregationFunction, str]] = ..., function_params: Optional[Union[_aggregation_function__client_pb2.AggregationFunctionParams, Mapping]] = ..., window: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., time_window: Optional[Union[_time_window__client_pb2.TimeWindow, Mapping]] = ..., batch_sawtooth_tile_size: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., description: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ...) -> None: ...

class Attribute(_message.Message):
    __slots__ = ["column", "description", "tags"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    COLUMN_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    column: _schema__client_pb2.Field
    description: str
    tags: _containers.ScalarMap[str, str]
    def __init__(self, column: Optional[Union[_schema__client_pb2.Field, Mapping]] = ..., description: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ...) -> None: ...

class BigtableOnlineStore(_message.Message):
    __slots__ = ["enabled", "instance_id", "project_id"]
    ENABLED_FIELD_NUMBER: ClassVar[int]
    INSTANCE_ID_FIELD_NUMBER: ClassVar[int]
    PROJECT_ID_FIELD_NUMBER: ClassVar[int]
    enabled: bool
    instance_id: str
    project_id: str
    def __init__(self, enabled: bool = ..., project_id: Optional[str] = ..., instance_id: Optional[str] = ...) -> None: ...

class DataQualityConfig(_message.Message):
    __slots__ = ["data_quality_enabled", "skip_default_expectations"]
    DATA_QUALITY_ENABLED_FIELD_NUMBER: ClassVar[int]
    SKIP_DEFAULT_EXPECTATIONS_FIELD_NUMBER: ClassVar[int]
    data_quality_enabled: bool
    skip_default_expectations: bool
    def __init__(self, data_quality_enabled: bool = ..., skip_default_expectations: bool = ...) -> None: ...

class DeltaOfflineStoreParams(_message.Message):
    __slots__ = ["time_partition_size", "version"]
    TIME_PARTITION_SIZE_FIELD_NUMBER: ClassVar[int]
    VERSION_FIELD_NUMBER: ClassVar[int]
    time_partition_size: _duration_pb2.Duration
    version: DeltaOfflineStoreVersion
    def __init__(self, time_partition_size: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., version: Optional[Union[DeltaOfflineStoreVersion, str]] = ...) -> None: ...

class DynamoDbOnlineStore(_message.Message):
    __slots__ = ["cross_account_external_id", "cross_account_intermediate_role_arn", "cross_account_role_arn", "dbfs_credentials_path", "enabled"]
    CROSS_ACCOUNT_EXTERNAL_ID_FIELD_NUMBER: ClassVar[int]
    CROSS_ACCOUNT_INTERMEDIATE_ROLE_ARN_FIELD_NUMBER: ClassVar[int]
    CROSS_ACCOUNT_ROLE_ARN_FIELD_NUMBER: ClassVar[int]
    DBFS_CREDENTIALS_PATH_FIELD_NUMBER: ClassVar[int]
    ENABLED_FIELD_NUMBER: ClassVar[int]
    cross_account_external_id: str
    cross_account_intermediate_role_arn: str
    cross_account_role_arn: str
    dbfs_credentials_path: str
    enabled: bool
    def __init__(self, cross_account_role_arn: Optional[str] = ..., cross_account_external_id: Optional[str] = ..., cross_account_intermediate_role_arn: Optional[str] = ..., enabled: bool = ..., dbfs_credentials_path: Optional[str] = ...) -> None: ...

class Embedding(_message.Message):
    __slots__ = ["description", "input_column_name", "model", "output_column_name", "tags"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    INPUT_COLUMN_NAME_FIELD_NUMBER: ClassVar[int]
    MODEL_FIELD_NUMBER: ClassVar[int]
    OUTPUT_COLUMN_NAME_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    description: str
    input_column_name: str
    model: str
    output_column_name: str
    tags: _containers.ScalarMap[str, str]
    def __init__(self, input_column_name: Optional[str] = ..., output_column_name: Optional[str] = ..., model: Optional[str] = ..., description: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ...) -> None: ...

class FeaturePublishOfflineStoreConfig(_message.Message):
    __slots__ = ["publish_full_features", "publish_start_time"]
    PUBLISH_FULL_FEATURES_FIELD_NUMBER: ClassVar[int]
    PUBLISH_START_TIME_FIELD_NUMBER: ClassVar[int]
    publish_full_features: bool
    publish_start_time: _timestamp_pb2.Timestamp
    def __init__(self, publish_full_features: bool = ..., publish_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class FeatureTable(_message.Message):
    __slots__ = ["attributes", "offline_enabled", "online_enabled", "serving_ttl"]
    ATTRIBUTES_FIELD_NUMBER: ClassVar[int]
    OFFLINE_ENABLED_FIELD_NUMBER: ClassVar[int]
    ONLINE_ENABLED_FIELD_NUMBER: ClassVar[int]
    SERVING_TTL_FIELD_NUMBER: ClassVar[int]
    attributes: _containers.RepeatedCompositeFieldContainer[Attribute]
    offline_enabled: bool
    online_enabled: bool
    serving_ttl: _duration_pb2.Duration
    def __init__(self, online_enabled: bool = ..., offline_enabled: bool = ..., serving_ttl: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., attributes: Optional[Iterable[Union[Attribute, Mapping]]] = ...) -> None: ...

class FeatureView(_message.Message):
    __slots__ = ["batch_compute_mode", "batch_trigger", "cache_config", "context_parameter_name", "data_quality_config", "enrichments", "entity_ids", "fco_metadata", "feature_store_format_version", "feature_table", "feature_view_id", "framework_version", "fw_version", "join_keys", "materialization_enabled", "materialization_params", "materialization_state_transitions", "monitoring_params", "online_serving_index", "options", "pipeline", "prompt", "realtime_feature_view", "schemas", "snowflake_data", "temporal", "temporal_aggregate", "timestamp_key", "validation_args", "web_url"]
    class OptionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    BATCH_COMPUTE_MODE_FIELD_NUMBER: ClassVar[int]
    BATCH_TRIGGER_FIELD_NUMBER: ClassVar[int]
    CACHE_CONFIG_FIELD_NUMBER: ClassVar[int]
    CONTEXT_PARAMETER_NAME_FIELD_NUMBER: ClassVar[int]
    DATA_QUALITY_CONFIG_FIELD_NUMBER: ClassVar[int]
    ENRICHMENTS_FIELD_NUMBER: ClassVar[int]
    ENTITY_IDS_FIELD_NUMBER: ClassVar[int]
    FCO_METADATA_FIELD_NUMBER: ClassVar[int]
    FEATURE_STORE_FORMAT_VERSION_FIELD_NUMBER: ClassVar[int]
    FEATURE_TABLE_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_ID_FIELD_NUMBER: ClassVar[int]
    FRAMEWORK_VERSION_FIELD_NUMBER: ClassVar[int]
    FW_VERSION_FIELD_NUMBER: ClassVar[int]
    JOIN_KEYS_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_ENABLED_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_PARAMS_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_STATE_TRANSITIONS_FIELD_NUMBER: ClassVar[int]
    MONITORING_PARAMS_FIELD_NUMBER: ClassVar[int]
    ONLINE_SERVING_INDEX_FIELD_NUMBER: ClassVar[int]
    OPTIONS_FIELD_NUMBER: ClassVar[int]
    PIPELINE_FIELD_NUMBER: ClassVar[int]
    PROMPT_FIELD_NUMBER: ClassVar[int]
    REALTIME_FEATURE_VIEW_FIELD_NUMBER: ClassVar[int]
    SCHEMAS_FIELD_NUMBER: ClassVar[int]
    SNOWFLAKE_DATA_FIELD_NUMBER: ClassVar[int]
    TEMPORAL_AGGREGATE_FIELD_NUMBER: ClassVar[int]
    TEMPORAL_FIELD_NUMBER: ClassVar[int]
    TIMESTAMP_KEY_FIELD_NUMBER: ClassVar[int]
    VALIDATION_ARGS_FIELD_NUMBER: ClassVar[int]
    WEB_URL_FIELD_NUMBER: ClassVar[int]
    batch_compute_mode: _compute_mode__client_pb2.BatchComputeMode
    batch_trigger: _feature_view__client_pb2.BatchTriggerType
    cache_config: FeatureViewCacheConfig
    context_parameter_name: str
    data_quality_config: DataQualityConfig
    enrichments: FeatureViewEnrichments
    entity_ids: _containers.RepeatedCompositeFieldContainer[_id__client_pb2.Id]
    fco_metadata: _fco_metadata__client_pb2.FcoMetadata
    feature_store_format_version: int
    feature_table: FeatureTable
    feature_view_id: _id__client_pb2.Id
    framework_version: int
    fw_version: _framework_version__client_pb2.FrameworkVersion
    join_keys: _containers.RepeatedScalarFieldContainer[str]
    materialization_enabled: bool
    materialization_params: NewMaterializationParams
    materialization_state_transitions: _containers.RepeatedCompositeFieldContainer[MaterializationStateTransition]
    monitoring_params: MonitoringParams
    online_serving_index: OnlineServingIndex
    options: _containers.ScalarMap[str, str]
    pipeline: _pipeline__client_pb2.Pipeline
    prompt: Prompt
    realtime_feature_view: RealtimeFeatureView
    schemas: FeatureViewSchemas
    snowflake_data: SnowflakeData
    temporal: Temporal
    temporal_aggregate: TemporalAggregate
    timestamp_key: str
    validation_args: _validator__client_pb2.FeatureViewValidationArgs
    web_url: str
    def __init__(self, feature_view_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., fco_metadata: Optional[Union[_fco_metadata__client_pb2.FcoMetadata, Mapping]] = ..., entity_ids: Optional[Iterable[Union[_id__client_pb2.Id, Mapping]]] = ..., join_keys: Optional[Iterable[str]] = ..., schemas: Optional[Union[FeatureViewSchemas, Mapping]] = ..., enrichments: Optional[Union[FeatureViewEnrichments, Mapping]] = ..., temporal_aggregate: Optional[Union[TemporalAggregate, Mapping]] = ..., temporal: Optional[Union[Temporal, Mapping]] = ..., realtime_feature_view: Optional[Union[RealtimeFeatureView, Mapping]] = ..., feature_table: Optional[Union[FeatureTable, Mapping]] = ..., prompt: Optional[Union[Prompt, Mapping]] = ..., timestamp_key: Optional[str] = ..., online_serving_index: Optional[Union[OnlineServingIndex, Mapping]] = ..., pipeline: Optional[Union[_pipeline__client_pb2.Pipeline, Mapping]] = ..., materialization_params: Optional[Union[NewMaterializationParams, Mapping]] = ..., materialization_enabled: bool = ..., materialization_state_transitions: Optional[Iterable[Union[MaterializationStateTransition, Mapping]]] = ..., monitoring_params: Optional[Union[MonitoringParams, Mapping]] = ..., feature_store_format_version: Optional[int] = ..., snowflake_data: Optional[Union[SnowflakeData, Mapping]] = ..., framework_version: Optional[int] = ..., fw_version: Optional[Union[_framework_version__client_pb2.FrameworkVersion, str]] = ..., web_url: Optional[str] = ..., batch_trigger: Optional[Union[_feature_view__client_pb2.BatchTriggerType, str]] = ..., validation_args: Optional[Union[_validator__client_pb2.FeatureViewValidationArgs, Mapping]] = ..., data_quality_config: Optional[Union[DataQualityConfig, Mapping]] = ..., options: Optional[Mapping[str, str]] = ..., batch_compute_mode: Optional[Union[_compute_mode__client_pb2.BatchComputeMode, str]] = ..., cache_config: Optional[Union[FeatureViewCacheConfig, Mapping]] = ..., context_parameter_name: Optional[str] = ...) -> None: ...

class FeatureViewCacheConfig(_message.Message):
    __slots__ = ["cache_group_name", "max_age_jitter", "max_age_seconds", "namespace", "remapped_join_keys"]
    CACHE_GROUP_NAME_FIELD_NUMBER: ClassVar[int]
    MAX_AGE_JITTER_FIELD_NUMBER: ClassVar[int]
    MAX_AGE_SECONDS_FIELD_NUMBER: ClassVar[int]
    NAMESPACE_FIELD_NUMBER: ClassVar[int]
    REMAPPED_JOIN_KEYS_FIELD_NUMBER: ClassVar[int]
    cache_group_name: str
    max_age_jitter: _duration_pb2.Duration
    max_age_seconds: _duration_pb2.Duration
    namespace: str
    remapped_join_keys: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, namespace: Optional[str] = ..., cache_group_name: Optional[str] = ..., max_age_seconds: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., max_age_jitter: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., remapped_join_keys: Optional[Iterable[str]] = ...) -> None: ...

class FeatureViewEnrichments(_message.Message):
    __slots__ = ["fp_materialization"]
    FP_MATERIALIZATION_FIELD_NUMBER: ClassVar[int]
    fp_materialization: _fv_materialization__client_pb2.FvMaterialization
    def __init__(self, fp_materialization: Optional[Union[_fv_materialization__client_pb2.FvMaterialization, Mapping]] = ...) -> None: ...

class FeatureViewSchemas(_message.Message):
    __slots__ = ["is_explicit_view_schema", "materialization_schema", "online_batch_table_format", "view_schema"]
    IS_EXPLICIT_VIEW_SCHEMA_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_SCHEMA_FIELD_NUMBER: ClassVar[int]
    ONLINE_BATCH_TABLE_FORMAT_FIELD_NUMBER: ClassVar[int]
    VIEW_SCHEMA_FIELD_NUMBER: ClassVar[int]
    is_explicit_view_schema: bool
    materialization_schema: _schema__client_pb2.Schema
    online_batch_table_format: _schema__client_pb2.OnlineBatchTableFormat
    view_schema: _schema__client_pb2.Schema
    def __init__(self, view_schema: Optional[Union[_schema__client_pb2.Schema, Mapping]] = ..., is_explicit_view_schema: bool = ..., materialization_schema: Optional[Union[_schema__client_pb2.Schema, Mapping]] = ..., online_batch_table_format: Optional[Union[_schema__client_pb2.OnlineBatchTableFormat, Mapping]] = ...) -> None: ...

class Inference(_message.Message):
    __slots__ = ["description", "input_columns", "model_artifact", "output_column", "tags"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    INPUT_COLUMNS_FIELD_NUMBER: ClassVar[int]
    MODEL_ARTIFACT_FIELD_NUMBER: ClassVar[int]
    OUTPUT_COLUMN_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    description: str
    input_columns: _containers.RepeatedCompositeFieldContainer[_schema__client_pb2.Column]
    model_artifact: _model_artifact_service__client_pb2.ModelArtifactInfo
    output_column: _schema__client_pb2.Column
    tags: _containers.ScalarMap[str, str]
    def __init__(self, input_columns: Optional[Iterable[Union[_schema__client_pb2.Column, Mapping]]] = ..., output_column: Optional[Union[_schema__client_pb2.Column, Mapping]] = ..., model_artifact: Optional[Union[_model_artifact_service__client_pb2.ModelArtifactInfo, Mapping]] = ..., description: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ...) -> None: ...

class MaterializationStateTransition(_message.Message):
    __slots__ = ["feature_start_timestamp", "force_stream_job_restart", "materialization_serial_version", "offline_enabled", "online_enabled", "tecton_runtime_version", "timestamp"]
    FEATURE_START_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    FORCE_STREAM_JOB_RESTART_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_SERIAL_VERSION_FIELD_NUMBER: ClassVar[int]
    OFFLINE_ENABLED_FIELD_NUMBER: ClassVar[int]
    ONLINE_ENABLED_FIELD_NUMBER: ClassVar[int]
    TECTON_RUNTIME_VERSION_FIELD_NUMBER: ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    feature_start_timestamp: _timestamp_pb2.Timestamp
    force_stream_job_restart: bool
    materialization_serial_version: int
    offline_enabled: bool
    online_enabled: bool
    tecton_runtime_version: str
    timestamp: _timestamp_pb2.Timestamp
    def __init__(self, timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., online_enabled: bool = ..., offline_enabled: bool = ..., feature_start_timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., materialization_serial_version: Optional[int] = ..., force_stream_job_restart: bool = ..., tecton_runtime_version: Optional[str] = ...) -> None: ...

class MonitoringParams(_message.Message):
    __slots__ = ["alert_email", "expected_feature_freshness", "grace_period_seconds", "monitor_freshness", "user_specified"]
    ALERT_EMAIL_FIELD_NUMBER: ClassVar[int]
    EXPECTED_FEATURE_FRESHNESS_FIELD_NUMBER: ClassVar[int]
    GRACE_PERIOD_SECONDS_FIELD_NUMBER: ClassVar[int]
    MONITOR_FRESHNESS_FIELD_NUMBER: ClassVar[int]
    USER_SPECIFIED_FIELD_NUMBER: ClassVar[int]
    alert_email: str
    expected_feature_freshness: _duration_pb2.Duration
    grace_period_seconds: int
    monitor_freshness: bool
    user_specified: bool
    def __init__(self, user_specified: bool = ..., monitor_freshness: bool = ..., expected_feature_freshness: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., alert_email: Optional[str] = ..., grace_period_seconds: Optional[int] = ...) -> None: ...

class NewMaterializationParams(_message.Message):
    __slots__ = ["aggregation_leading_edge", "batch_materialization", "compaction_enabled", "environment", "feature_publish_offline_store_config", "feature_start_timestamp", "manual_trigger_backfill_end_timestamp", "materialization_start_timestamp", "max_backfill_interval", "max_source_data_delay", "offline_store_config", "offline_store_params", "online_backfill_load_type", "online_store_params", "output_stream", "schedule_interval", "stream_materialization", "stream_tile_size", "stream_tiling_enabled", "tecton_materialization_runtime", "time_range_policy", "writes_to_offline_store", "writes_to_online_store"]
    AGGREGATION_LEADING_EDGE_FIELD_NUMBER: ClassVar[int]
    BATCH_MATERIALIZATION_FIELD_NUMBER: ClassVar[int]
    COMPACTION_ENABLED_FIELD_NUMBER: ClassVar[int]
    ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    FEATURE_PUBLISH_OFFLINE_STORE_CONFIG_FIELD_NUMBER: ClassVar[int]
    FEATURE_START_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    MANUAL_TRIGGER_BACKFILL_END_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_START_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    MAX_BACKFILL_INTERVAL_FIELD_NUMBER: ClassVar[int]
    MAX_SOURCE_DATA_DELAY_FIELD_NUMBER: ClassVar[int]
    OFFLINE_STORE_CONFIG_FIELD_NUMBER: ClassVar[int]
    OFFLINE_STORE_PARAMS_FIELD_NUMBER: ClassVar[int]
    ONLINE_BACKFILL_LOAD_TYPE_FIELD_NUMBER: ClassVar[int]
    ONLINE_STORE_PARAMS_FIELD_NUMBER: ClassVar[int]
    OUTPUT_STREAM_FIELD_NUMBER: ClassVar[int]
    SCHEDULE_INTERVAL_FIELD_NUMBER: ClassVar[int]
    STREAM_MATERIALIZATION_FIELD_NUMBER: ClassVar[int]
    STREAM_TILE_SIZE_FIELD_NUMBER: ClassVar[int]
    STREAM_TILING_ENABLED_FIELD_NUMBER: ClassVar[int]
    TECTON_MATERIALIZATION_RUNTIME_FIELD_NUMBER: ClassVar[int]
    TIME_RANGE_POLICY_FIELD_NUMBER: ClassVar[int]
    WRITES_TO_OFFLINE_STORE_FIELD_NUMBER: ClassVar[int]
    WRITES_TO_ONLINE_STORE_FIELD_NUMBER: ClassVar[int]
    aggregation_leading_edge: _feature_view__client_pb2.AggregationLeadingEdge
    batch_materialization: _feature_view__client_pb2.ClusterConfig
    compaction_enabled: bool
    environment: str
    feature_publish_offline_store_config: FeaturePublishOfflineStoreConfig
    feature_start_timestamp: _timestamp_pb2.Timestamp
    manual_trigger_backfill_end_timestamp: _timestamp_pb2.Timestamp
    materialization_start_timestamp: _timestamp_pb2.Timestamp
    max_backfill_interval: _duration_pb2.Duration
    max_source_data_delay: _duration_pb2.Duration
    offline_store_config: _feature_view__client_pb2.OfflineFeatureStoreConfig
    offline_store_params: OfflineStoreParams
    online_backfill_load_type: _fv_materialization__client_pb2.OnlineBackfillLoadType
    online_store_params: OnlineStoreParams
    output_stream: _feature_view__client_pb2.OutputStream
    schedule_interval: _duration_pb2.Duration
    stream_materialization: _feature_view__client_pb2.ClusterConfig
    stream_tile_size: _duration_pb2.Duration
    stream_tiling_enabled: bool
    tecton_materialization_runtime: str
    time_range_policy: MaterializationTimeRangePolicy
    writes_to_offline_store: bool
    writes_to_online_store: bool
    def __init__(self, schedule_interval: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., materialization_start_timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_start_timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., manual_trigger_backfill_end_timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., max_backfill_interval: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., writes_to_online_store: bool = ..., writes_to_offline_store: bool = ..., offline_store_config: Optional[Union[_feature_view__client_pb2.OfflineFeatureStoreConfig, Mapping]] = ..., offline_store_params: Optional[Union[OfflineStoreParams, Mapping]] = ..., batch_materialization: Optional[Union[_feature_view__client_pb2.ClusterConfig, Mapping]] = ..., stream_materialization: Optional[Union[_feature_view__client_pb2.ClusterConfig, Mapping]] = ..., max_source_data_delay: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., online_store_params: Optional[Union[OnlineStoreParams, Mapping]] = ..., output_stream: Optional[Union[_feature_view__client_pb2.OutputStream, Mapping]] = ..., time_range_policy: Optional[Union[MaterializationTimeRangePolicy, str]] = ..., online_backfill_load_type: Optional[Union[_fv_materialization__client_pb2.OnlineBackfillLoadType, str]] = ..., tecton_materialization_runtime: Optional[str] = ..., feature_publish_offline_store_config: Optional[Union[FeaturePublishOfflineStoreConfig, Mapping]] = ..., compaction_enabled: bool = ..., stream_tiling_enabled: bool = ..., environment: Optional[str] = ..., stream_tile_size: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., aggregation_leading_edge: Optional[Union[_feature_view__client_pb2.AggregationLeadingEdge, str]] = ...) -> None: ...

class OfflineStoreParams(_message.Message):
    __slots__ = ["delta", "parquet"]
    DELTA_FIELD_NUMBER: ClassVar[int]
    PARQUET_FIELD_NUMBER: ClassVar[int]
    delta: DeltaOfflineStoreParams
    parquet: ParquetOfflineStoreParams
    def __init__(self, parquet: Optional[Union[ParquetOfflineStoreParams, Mapping]] = ..., delta: Optional[Union[DeltaOfflineStoreParams, Mapping]] = ...) -> None: ...

class OnlineServingIndex(_message.Message):
    __slots__ = ["join_keys"]
    JOIN_KEYS_FIELD_NUMBER: ClassVar[int]
    join_keys: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, join_keys: Optional[Iterable[str]] = ...) -> None: ...

class OnlineStoreParams(_message.Message):
    __slots__ = ["bigtable", "dynamo", "redis"]
    BIGTABLE_FIELD_NUMBER: ClassVar[int]
    DYNAMO_FIELD_NUMBER: ClassVar[int]
    REDIS_FIELD_NUMBER: ClassVar[int]
    bigtable: BigtableOnlineStore
    dynamo: DynamoDbOnlineStore
    redis: RedisOnlineStore
    def __init__(self, dynamo: Optional[Union[DynamoDbOnlineStore, Mapping]] = ..., redis: Optional[Union[RedisOnlineStore, Mapping]] = ..., bigtable: Optional[Union[BigtableOnlineStore, Mapping]] = ...) -> None: ...

class ParquetOfflineStoreParams(_message.Message):
    __slots__ = ["time_partition_size", "version"]
    TIME_PARTITION_SIZE_FIELD_NUMBER: ClassVar[int]
    VERSION_FIELD_NUMBER: ClassVar[int]
    time_partition_size: _duration_pb2.Duration
    version: ParquetOfflineStoreVersion
    def __init__(self, time_partition_size: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., version: Optional[Union[ParquetOfflineStoreVersion, str]] = ...) -> None: ...

class Prompt(_message.Message):
    __slots__ = ["attributes", "environment"]
    ATTRIBUTES_FIELD_NUMBER: ClassVar[int]
    ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    attributes: _containers.RepeatedCompositeFieldContainer[Attribute]
    environment: _realtime_compute__client_pb2.RemoteFunctionComputeConfig
    def __init__(self, environment: Optional[Union[_realtime_compute__client_pb2.RemoteFunctionComputeConfig, Mapping]] = ..., attributes: Optional[Iterable[Union[Attribute, Mapping]]] = ...) -> None: ...

class RealtimeFeatureView(_message.Message):
    __slots__ = ["attributes", "no_op", "supported_environments"]
    ATTRIBUTES_FIELD_NUMBER: ClassVar[int]
    NO_OP_FIELD_NUMBER: ClassVar[int]
    SUPPORTED_ENVIRONMENTS_FIELD_NUMBER: ClassVar[int]
    attributes: _containers.RepeatedCompositeFieldContainer[Attribute]
    no_op: bool
    supported_environments: _containers.RepeatedCompositeFieldContainer[_realtime_compute__client_pb2.RemoteFunctionComputeConfig]
    def __init__(self, no_op: bool = ..., supported_environments: Optional[Iterable[Union[_realtime_compute__client_pb2.RemoteFunctionComputeConfig, Mapping]]] = ..., attributes: Optional[Iterable[Union[Attribute, Mapping]]] = ...) -> None: ...

class RedisOnlineStore(_message.Message):
    __slots__ = ["authentication_token", "enabled", "inject_host_sni", "primary_endpoint", "tls_enabled"]
    AUTHENTICATION_TOKEN_FIELD_NUMBER: ClassVar[int]
    ENABLED_FIELD_NUMBER: ClassVar[int]
    INJECT_HOST_SNI_FIELD_NUMBER: ClassVar[int]
    PRIMARY_ENDPOINT_FIELD_NUMBER: ClassVar[int]
    TLS_ENABLED_FIELD_NUMBER: ClassVar[int]
    authentication_token: str
    enabled: bool
    inject_host_sni: bool
    primary_endpoint: str
    tls_enabled: bool
    def __init__(self, primary_endpoint: Optional[str] = ..., authentication_token: Optional[str] = ..., tls_enabled: bool = ..., enabled: bool = ..., inject_host_sni: bool = ...) -> None: ...

class SecondaryKeyOutputColumn(_message.Message):
    __slots__ = ["name", "time_window"]
    NAME_FIELD_NUMBER: ClassVar[int]
    TIME_WINDOW_FIELD_NUMBER: ClassVar[int]
    name: str
    time_window: _time_window__client_pb2.TimeWindow
    def __init__(self, time_window: Optional[Union[_time_window__client_pb2.TimeWindow, Mapping]] = ..., name: Optional[str] = ...) -> None: ...

class SnowflakeData(_message.Message):
    __slots__ = ["snowflake_view_name"]
    SNOWFLAKE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    snowflake_view_name: str
    def __init__(self, snowflake_view_name: Optional[str] = ...) -> None: ...

class Temporal(_message.Message):
    __slots__ = ["attributes", "backfill_config", "data_source_type", "embeddings", "incremental_backfills", "inferences", "is_continuous", "serving_ttl"]
    ATTRIBUTES_FIELD_NUMBER: ClassVar[int]
    BACKFILL_CONFIG_FIELD_NUMBER: ClassVar[int]
    DATA_SOURCE_TYPE_FIELD_NUMBER: ClassVar[int]
    EMBEDDINGS_FIELD_NUMBER: ClassVar[int]
    INCREMENTAL_BACKFILLS_FIELD_NUMBER: ClassVar[int]
    INFERENCES_FIELD_NUMBER: ClassVar[int]
    IS_CONTINUOUS_FIELD_NUMBER: ClassVar[int]
    SERVING_TTL_FIELD_NUMBER: ClassVar[int]
    attributes: _containers.RepeatedCompositeFieldContainer[Attribute]
    backfill_config: _feature_view__client_pb2.BackfillConfig
    data_source_type: _data_source_type__client_pb2.DataSourceType
    embeddings: _containers.RepeatedCompositeFieldContainer[Embedding]
    incremental_backfills: bool
    inferences: _containers.RepeatedCompositeFieldContainer[Inference]
    is_continuous: bool
    serving_ttl: _duration_pb2.Duration
    def __init__(self, serving_ttl: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., data_source_type: Optional[Union[_data_source_type__client_pb2.DataSourceType, str]] = ..., backfill_config: Optional[Union[_feature_view__client_pb2.BackfillConfig, Mapping]] = ..., incremental_backfills: bool = ..., is_continuous: bool = ..., embeddings: Optional[Iterable[Union[Embedding, Mapping]]] = ..., inferences: Optional[Iterable[Union[Inference, Mapping]]] = ..., attributes: Optional[Iterable[Union[Attribute, Mapping]]] = ...) -> None: ...

class TemporalAggregate(_message.Message):
    __slots__ = ["aggregation_secondary_key", "data_source_type", "features", "is_continuous", "secondary_key_output_columns", "slide_interval", "slide_interval_string"]
    AGGREGATION_SECONDARY_KEY_FIELD_NUMBER: ClassVar[int]
    DATA_SOURCE_TYPE_FIELD_NUMBER: ClassVar[int]
    FEATURES_FIELD_NUMBER: ClassVar[int]
    IS_CONTINUOUS_FIELD_NUMBER: ClassVar[int]
    SECONDARY_KEY_OUTPUT_COLUMNS_FIELD_NUMBER: ClassVar[int]
    SLIDE_INTERVAL_FIELD_NUMBER: ClassVar[int]
    SLIDE_INTERVAL_STRING_FIELD_NUMBER: ClassVar[int]
    aggregation_secondary_key: str
    data_source_type: _data_source_type__client_pb2.DataSourceType
    features: _containers.RepeatedCompositeFieldContainer[Aggregate]
    is_continuous: bool
    secondary_key_output_columns: _containers.RepeatedCompositeFieldContainer[SecondaryKeyOutputColumn]
    slide_interval: _duration_pb2.Duration
    slide_interval_string: str
    def __init__(self, slide_interval: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., slide_interval_string: Optional[str] = ..., features: Optional[Iterable[Union[Aggregate, Mapping]]] = ..., is_continuous: bool = ..., data_source_type: Optional[Union[_data_source_type__client_pb2.DataSourceType, str]] = ..., aggregation_secondary_key: Optional[str] = ..., secondary_key_output_columns: Optional[Iterable[Union[SecondaryKeyOutputColumn, Mapping]]] = ...) -> None: ...

class TrailingTimeWindowAggregation(_message.Message):
    __slots__ = ["aggregation_slide_period", "features", "is_continuous", "time_key"]
    AGGREGATION_SLIDE_PERIOD_FIELD_NUMBER: ClassVar[int]
    FEATURES_FIELD_NUMBER: ClassVar[int]
    IS_CONTINUOUS_FIELD_NUMBER: ClassVar[int]
    TIME_KEY_FIELD_NUMBER: ClassVar[int]
    aggregation_slide_period: _duration_pb2.Duration
    features: _containers.RepeatedCompositeFieldContainer[Aggregate]
    is_continuous: bool
    time_key: str
    def __init__(self, time_key: Optional[str] = ..., aggregation_slide_period: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., features: Optional[Iterable[Union[Aggregate, Mapping]]] = ..., is_continuous: bool = ...) -> None: ...

class ParquetOfflineStoreVersion(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class DeltaOfflineStoreVersion(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class MaterializationTimeRangePolicy(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from tecton_proto.auth import service__client_pb2 as _service__client_pb2
from tecton_proto.common import container_image__client_pb2 as _container_image__client_pb2
from tecton_proto.data import remote_compute_environment__client_pb2 as _remote_compute_environment__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Text, Union

DESCRIPTOR: _descriptor.FileDescriptor

class CompletePackagesUploadRequest(_message.Message):
    __slots__ = ["upload_info"]
    UPLOAD_INFO_FIELD_NUMBER: ClassVar[int]
    upload_info: _remote_compute_environment__client_pb2.RemoteEnvironmentUploadInfo
    def __init__(self, upload_info: Optional[Union[_remote_compute_environment__client_pb2.RemoteEnvironmentUploadInfo, Mapping]] = ...) -> None: ...

class CompletePackagesUploadResponse(_message.Message):
    __slots__ = ["storage_location"]
    STORAGE_LOCATION_FIELD_NUMBER: ClassVar[int]
    storage_location: str
    def __init__(self, storage_location: Optional[str] = ...) -> None: ...

class CreateRemoteEnvironmentRequest(_message.Message):
    __slots__ = ["description", "id", "image_info", "name", "online_provisioned", "python_version", "requirements", "resolved_requirements", "rift_materialization_runtime_version", "s3_wheels_location", "sdk_version", "transform_runtime_version"]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    ID_FIELD_NUMBER: ClassVar[int]
    IMAGE_INFO_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    ONLINE_PROVISIONED_FIELD_NUMBER: ClassVar[int]
    PYTHON_VERSION_FIELD_NUMBER: ClassVar[int]
    REQUIREMENTS_FIELD_NUMBER: ClassVar[int]
    RESOLVED_REQUIREMENTS_FIELD_NUMBER: ClassVar[int]
    RIFT_MATERIALIZATION_RUNTIME_VERSION_FIELD_NUMBER: ClassVar[int]
    S3_WHEELS_LOCATION_FIELD_NUMBER: ClassVar[int]
    SDK_VERSION_FIELD_NUMBER: ClassVar[int]
    TRANSFORM_RUNTIME_VERSION_FIELD_NUMBER: ClassVar[int]
    description: str
    id: str
    image_info: _container_image__client_pb2.ContainerImage
    name: str
    online_provisioned: bool
    python_version: str
    requirements: str
    resolved_requirements: str
    rift_materialization_runtime_version: str
    s3_wheels_location: str
    sdk_version: str
    transform_runtime_version: str
    def __init__(self, name: Optional[str] = ..., description: Optional[str] = ..., image_info: Optional[Union[_container_image__client_pb2.ContainerImage, Mapping]] = ..., id: Optional[str] = ..., python_version: Optional[str] = ..., requirements: Optional[str] = ..., resolved_requirements: Optional[str] = ..., s3_wheels_location: Optional[str] = ..., transform_runtime_version: Optional[str] = ..., rift_materialization_runtime_version: Optional[str] = ..., sdk_version: Optional[str] = ..., online_provisioned: bool = ...) -> None: ...

class CreateRemoteEnvironmentResponse(_message.Message):
    __slots__ = ["remote_environment"]
    REMOTE_ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    remote_environment: _remote_compute_environment__client_pb2.RemoteComputeEnvironment
    def __init__(self, remote_environment: Optional[Union[_remote_compute_environment__client_pb2.RemoteComputeEnvironment, Mapping]] = ...) -> None: ...

class DeleteRemoteEnvironmentsRequest(_message.Message):
    __slots__ = ["ids"]
    IDS_FIELD_NUMBER: ClassVar[int]
    ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, ids: Optional[Iterable[str]] = ...) -> None: ...

class DeleteRemoteEnvironmentsResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class GetDependentFeatureServicesRequest(_message.Message):
    __slots__ = ["environment_id"]
    ENVIRONMENT_ID_FIELD_NUMBER: ClassVar[int]
    environment_id: str
    def __init__(self, environment_id: Optional[str] = ...) -> None: ...

class GetDependentFeatureServicesResponse(_message.Message):
    __slots__ = ["dependent_feature_services"]
    DEPENDENT_FEATURE_SERVICES_FIELD_NUMBER: ClassVar[int]
    dependent_feature_services: _containers.RepeatedCompositeFieldContainer[_remote_compute_environment__client_pb2.DependentFeatureService]
    def __init__(self, dependent_feature_services: Optional[Iterable[Union[_remote_compute_environment__client_pb2.DependentFeatureService, Mapping]]] = ...) -> None: ...

class GetPackagesUploadUrlRequest(_message.Message):
    __slots__ = ["environment_id", "upload_part"]
    ENVIRONMENT_ID_FIELD_NUMBER: ClassVar[int]
    UPLOAD_PART_FIELD_NUMBER: ClassVar[int]
    environment_id: str
    upload_part: _remote_compute_environment__client_pb2.ObjectStoreUploadPart
    def __init__(self, environment_id: Optional[str] = ..., upload_part: Optional[Union[_remote_compute_environment__client_pb2.ObjectStoreUploadPart, Mapping]] = ...) -> None: ...

class GetPackagesUploadUrlResponse(_message.Message):
    __slots__ = ["upload_url"]
    UPLOAD_URL_FIELD_NUMBER: ClassVar[int]
    upload_url: str
    def __init__(self, upload_url: Optional[str] = ...) -> None: ...

class GetRemoteEnvironmentRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: ClassVar[int]
    id: str
    def __init__(self, id: Optional[str] = ...) -> None: ...

class GetRemoteEnvironmentResponse(_message.Message):
    __slots__ = ["remote_environment"]
    REMOTE_ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    remote_environment: _remote_compute_environment__client_pb2.RemoteComputeEnvironment
    def __init__(self, remote_environment: Optional[Union[_remote_compute_environment__client_pb2.RemoteComputeEnvironment, Mapping]] = ...) -> None: ...

class ListRemoteEnvironmentsRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListRemoteEnvironmentsResponse(_message.Message):
    __slots__ = ["remote_environments"]
    REMOTE_ENVIRONMENTS_FIELD_NUMBER: ClassVar[int]
    remote_environments: _containers.RepeatedCompositeFieldContainer[_remote_compute_environment__client_pb2.RemoteComputeEnvironment]
    def __init__(self, remote_environments: Optional[Iterable[Union[_remote_compute_environment__client_pb2.RemoteComputeEnvironment, Mapping]]] = ...) -> None: ...

class StartPackagesUploadRequest(_message.Message):
    __slots__ = ["environment_id"]
    ENVIRONMENT_ID_FIELD_NUMBER: ClassVar[int]
    environment_id: str
    def __init__(self, environment_id: Optional[str] = ...) -> None: ...

class StartPackagesUploadResponse(_message.Message):
    __slots__ = ["upload_info"]
    UPLOAD_INFO_FIELD_NUMBER: ClassVar[int]
    upload_info: _remote_compute_environment__client_pb2.RemoteEnvironmentUploadInfo
    def __init__(self, upload_info: Optional[Union[_remote_compute_environment__client_pb2.RemoteEnvironmentUploadInfo, Mapping]] = ...) -> None: ...

class UpdateRemoteEnvironmentRequest(_message.Message):
    __slots__ = ["id", "remote_function_version", "status"]
    ID_FIELD_NUMBER: ClassVar[int]
    REMOTE_FUNCTION_VERSION_FIELD_NUMBER: ClassVar[int]
    STATUS_FIELD_NUMBER: ClassVar[int]
    id: str
    remote_function_version: str
    status: _remote_compute_environment__client_pb2.RemoteEnvironmentStatus
    def __init__(self, id: Optional[str] = ..., remote_function_version: Optional[str] = ..., status: Optional[Union[_remote_compute_environment__client_pb2.RemoteEnvironmentStatus, str]] = ...) -> None: ...

class UpdateRemoteEnvironmentResponse(_message.Message):
    __slots__ = ["remote_environment"]
    REMOTE_ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    remote_environment: _remote_compute_environment__client_pb2.RemoteComputeEnvironment
    def __init__(self, remote_environment: Optional[Union[_remote_compute_environment__client_pb2.RemoteComputeEnvironment, Mapping]] = ...) -> None: ...



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class AccessControlListResponse(_message.Message):
    __slots__ = ["all_permissions", "display_name", "group_name", "service_principal_name", "user_name"]
    ALL_PERMISSIONS_FIELD_NUMBER: ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: ClassVar[int]
    GROUP_NAME_FIELD_NUMBER: ClassVar[int]
    SERVICE_PRINCIPAL_NAME_FIELD_NUMBER: ClassVar[int]
    USER_NAME_FIELD_NUMBER: ClassVar[int]
    all_permissions: _containers.RepeatedCompositeFieldContainer[PermissionObject]
    display_name: str
    group_name: str
    service_principal_name: str
    user_name: str
    def __init__(self, user_name: Optional[str] = ..., group_name: Optional[str] = ..., service_principal_name: Optional[str] = ..., display_name: Optional[str] = ..., all_permissions: Optional[Iterable[Union[PermissionObject, Mapping]]] = ...) -> None: ...

class GroupPermissionsObject(_message.Message):
    __slots__ = ["group_name", "permission_level", "service_principal_name", "user_name"]
    GROUP_NAME_FIELD_NUMBER: ClassVar[int]
    PERMISSION_LEVEL_FIELD_NUMBER: ClassVar[int]
    SERVICE_PRINCIPAL_NAME_FIELD_NUMBER: ClassVar[int]
    USER_NAME_FIELD_NUMBER: ClassVar[int]
    group_name: str
    permission_level: str
    service_principal_name: str
    user_name: str
    def __init__(self, group_name: Optional[str] = ..., permission_level: Optional[str] = ..., user_name: Optional[str] = ..., service_principal_name: Optional[str] = ...) -> None: ...

class PermissionObject(_message.Message):
    __slots__ = ["inherited", "inherited_from_object", "permission_level"]
    INHERITED_FIELD_NUMBER: ClassVar[int]
    INHERITED_FROM_OBJECT_FIELD_NUMBER: ClassVar[int]
    PERMISSION_LEVEL_FIELD_NUMBER: ClassVar[int]
    inherited: bool
    inherited_from_object: _containers.RepeatedScalarFieldContainer[str]
    permission_level: str
    def __init__(self, permission_level: Optional[str] = ..., inherited: bool = ..., inherited_from_object: Optional[Iterable[str]] = ...) -> None: ...

class PermissionsRequest(_message.Message):
    __slots__ = ["access_control_list"]
    ACCESS_CONTROL_LIST_FIELD_NUMBER: ClassVar[int]
    access_control_list: _containers.RepeatedCompositeFieldContainer[GroupPermissionsObject]
    def __init__(self, access_control_list: Optional[Iterable[Union[GroupPermissionsObject, Mapping]]] = ...) -> None: ...

class PermissionsResponse(_message.Message):
    __slots__ = ["access_control_list", "object_id", "object_type"]
    ACCESS_CONTROL_LIST_FIELD_NUMBER: ClassVar[int]
    OBJECT_ID_FIELD_NUMBER: ClassVar[int]
    OBJECT_TYPE_FIELD_NUMBER: ClassVar[int]
    access_control_list: _containers.RepeatedCompositeFieldContainer[AccessControlListResponse]
    object_id: str
    object_type: str
    def __init__(self, object_id: Optional[str] = ..., object_type: Optional[str] = ..., access_control_list: Optional[Iterable[Union[AccessControlListResponse, Mapping]]] = ...) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.args import diff_options__client_pb2 as _diff_options__client_pb2
from tecton_proto.args import feature_view__client_pb2 as _feature_view__client_pb2
from tecton_proto.common import compute_mode__client_pb2 as _compute_mode__client_pb2
from tecton_proto.common import fco_locator__client_pb2 as _fco_locator__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from tecton_proto.data import batch_data_source__client_pb2 as _batch_data_source__client_pb2
from tecton_proto.materialization import materialization_states__client_pb2 as _materialization_states__client_pb2
from tecton_proto.materialization import spark_cluster__client_pb2 as _spark_cluster__client_pb2
from tecton_proto.online_store_writer import config__client_pb2 as _config__client_pb2
from tecton_proto.snowflake import location__client_pb2 as _location__client_pb2
from tecton_proto.spark_api import jobs__client_pb2 as _jobs__client_pb2
from tecton_proto.spark_common import libraries__client_pb2 as _libraries__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class Attempt(_message.Message):
    __slots__ = ["has_legacy_materialization_task_attempt", "workflow_id"]
    HAS_LEGACY_MATERIALIZATION_TASK_ATTEMPT_FIELD_NUMBER: ClassVar[int]
    WORKFLOW_ID_FIELD_NUMBER: ClassVar[int]
    has_legacy_materialization_task_attempt: bool
    workflow_id: _id__client_pb2.Id
    def __init__(self, workflow_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., has_legacy_materialization_task_attempt: bool = ...) -> None: ...

class BatchMaterializationParameters(_message.Message):
    __slots__ = ["batch_compaction_enabled", "bootstraps_online_store", "create_online_table", "create_online_table_parameters", "feature_end_time", "feature_start_time", "is_overwrite", "is_overwrite_backfill", "read_from_offline_store_for_online_write", "task_dependencies", "tile_count", "window_end_time", "window_start_time", "write_to_offline_feature_store", "write_to_online_feature_store"]
    BATCH_COMPACTION_ENABLED_FIELD_NUMBER: ClassVar[int]
    BOOTSTRAPS_ONLINE_STORE_FIELD_NUMBER: ClassVar[int]
    CREATE_ONLINE_TABLE_FIELD_NUMBER: ClassVar[int]
    CREATE_ONLINE_TABLE_PARAMETERS_FIELD_NUMBER: ClassVar[int]
    FEATURE_END_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: ClassVar[int]
    IS_OVERWRITE_BACKFILL_FIELD_NUMBER: ClassVar[int]
    IS_OVERWRITE_FIELD_NUMBER: ClassVar[int]
    READ_FROM_OFFLINE_STORE_FOR_ONLINE_WRITE_FIELD_NUMBER: ClassVar[int]
    TASK_DEPENDENCIES_FIELD_NUMBER: ClassVar[int]
    TILE_COUNT_FIELD_NUMBER: ClassVar[int]
    WINDOW_END_TIME_FIELD_NUMBER: ClassVar[int]
    WINDOW_START_TIME_FIELD_NUMBER: ClassVar[int]
    WRITE_TO_OFFLINE_FEATURE_STORE_FIELD_NUMBER: ClassVar[int]
    WRITE_TO_ONLINE_FEATURE_STORE_FIELD_NUMBER: ClassVar[int]
    batch_compaction_enabled: bool
    bootstraps_online_store: bool
    create_online_table: bool
    create_online_table_parameters: CreateOnlineTableParameters
    feature_end_time: _timestamp_pb2.Timestamp
    feature_start_time: _timestamp_pb2.Timestamp
    is_overwrite: bool
    is_overwrite_backfill: bool
    read_from_offline_store_for_online_write: bool
    task_dependencies: MaterializationTaskDep
    tile_count: int
    window_end_time: _timestamp_pb2.Timestamp
    window_start_time: _timestamp_pb2.Timestamp
    write_to_offline_feature_store: bool
    write_to_online_feature_store: bool
    def __init__(self, window_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., window_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., tile_count: Optional[int] = ..., write_to_online_feature_store: bool = ..., write_to_offline_feature_store: bool = ..., is_overwrite_backfill: bool = ..., bootstraps_online_store: bool = ..., read_from_offline_store_for_online_write: bool = ..., create_online_table: bool = ..., create_online_table_parameters: Optional[Union[CreateOnlineTableParameters, Mapping]] = ..., batch_compaction_enabled: bool = ..., is_overwrite: bool = ..., task_dependencies: Optional[Union[MaterializationTaskDep, Mapping]] = ...) -> None: ...

class CanaryParams(_message.Message):
    __slots__ = ["canary_id", "canary_library_overrides", "canary_online_config", "canary_run_name", "canary_spark_cluster_environment", "canary_streaming_checkpoint_path"]
    CANARY_ID_FIELD_NUMBER: ClassVar[int]
    CANARY_LIBRARY_OVERRIDES_FIELD_NUMBER: ClassVar[int]
    CANARY_ONLINE_CONFIG_FIELD_NUMBER: ClassVar[int]
    CANARY_RUN_NAME_FIELD_NUMBER: ClassVar[int]
    CANARY_SPARK_CLUSTER_ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    CANARY_STREAMING_CHECKPOINT_PATH_FIELD_NUMBER: ClassVar[int]
    canary_id: str
    canary_library_overrides: _containers.RepeatedCompositeFieldContainer[_libraries__client_pb2.Library]
    canary_online_config: _config__client_pb2.OnlineStoreWriterConfiguration
    canary_run_name: str
    canary_spark_cluster_environment: _spark_cluster__client_pb2.SparkClusterEnvironment
    canary_streaming_checkpoint_path: str
    def __init__(self, canary_id: Optional[str] = ..., canary_online_config: Optional[Union[_config__client_pb2.OnlineStoreWriterConfiguration, Mapping]] = ..., canary_streaming_checkpoint_path: Optional[str] = ..., canary_library_overrides: Optional[Iterable[Union[_libraries__client_pb2.Library, Mapping]]] = ..., canary_spark_cluster_environment: Optional[Union[_spark_cluster__client_pb2.SparkClusterEnvironment, Mapping]] = ..., canary_run_name: Optional[str] = ...) -> None: ...

class CreateOnlineTableParameters(_message.Message):
    __slots__ = ["import_path_prefix"]
    IMPORT_PATH_PREFIX_FIELD_NUMBER: ClassVar[int]
    import_path_prefix: str
    def __init__(self, import_path_prefix: Optional[str] = ...) -> None: ...

class DatasetGenerationParameters(_message.Message):
    __slots__ = ["batch_config", "cluster_config", "dataset", "dataset_name", "datetime_range", "expected_schema", "extra_config", "feature_service", "feature_view", "from_source", "result_path", "spine"]
    class BatchConfigInput(_message.Message):
        __slots__ = ["batch_source", "end", "start"]
        BATCH_SOURCE_FIELD_NUMBER: ClassVar[int]
        END_FIELD_NUMBER: ClassVar[int]
        START_FIELD_NUMBER: ClassVar[int]
        batch_source: _batch_data_source__client_pb2.BatchDataSource
        end: _timestamp_pb2.Timestamp
        start: _timestamp_pb2.Timestamp
        def __init__(self, batch_source: Optional[Union[_batch_data_source__client_pb2.BatchDataSource, Mapping]] = ..., start: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., end: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...
    class DateTimeRangeInput(_message.Message):
        __slots__ = ["end", "entities_path", "max_lookback", "start"]
        END_FIELD_NUMBER: ClassVar[int]
        ENTITIES_PATH_FIELD_NUMBER: ClassVar[int]
        MAX_LOOKBACK_FIELD_NUMBER: ClassVar[int]
        START_FIELD_NUMBER: ClassVar[int]
        end: _timestamp_pb2.Timestamp
        entities_path: str
        max_lookback: _timestamp_pb2.Timestamp
        start: _timestamp_pb2.Timestamp
        def __init__(self, start: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., end: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., max_lookback: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., entities_path: Optional[str] = ...) -> None: ...
    class ExtraConfigEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    class SpineInput(_message.Message):
        __slots__ = ["path", "timestamp_key"]
        PATH_FIELD_NUMBER: ClassVar[int]
        TIMESTAMP_KEY_FIELD_NUMBER: ClassVar[int]
        path: str
        timestamp_key: str
        def __init__(self, path: Optional[str] = ..., timestamp_key: Optional[str] = ...) -> None: ...
    BATCH_CONFIG_FIELD_NUMBER: ClassVar[int]
    CLUSTER_CONFIG_FIELD_NUMBER: ClassVar[int]
    DATASET_FIELD_NUMBER: ClassVar[int]
    DATASET_NAME_FIELD_NUMBER: ClassVar[int]
    DATETIME_RANGE_FIELD_NUMBER: ClassVar[int]
    EXPECTED_SCHEMA_FIELD_NUMBER: ClassVar[int]
    EXTRA_CONFIG_FIELD_NUMBER: ClassVar[int]
    FEATURE_SERVICE_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_FIELD_NUMBER: ClassVar[int]
    FROM_SOURCE_FIELD_NUMBER: ClassVar[int]
    RESULT_PATH_FIELD_NUMBER: ClassVar[int]
    SPINE_FIELD_NUMBER: ClassVar[int]
    batch_config: DatasetGenerationParameters.BatchConfigInput
    cluster_config: _feature_view__client_pb2.ClusterConfig
    dataset: _fco_locator__client_pb2.IdFcoLocator
    dataset_name: str
    datetime_range: DatasetGenerationParameters.DateTimeRangeInput
    expected_schema: _schema__client_pb2.Schema
    extra_config: _containers.ScalarMap[str, str]
    feature_service: _fco_locator__client_pb2.IdFcoLocator
    feature_view: _fco_locator__client_pb2.IdFcoLocator
    from_source: bool
    result_path: str
    spine: DatasetGenerationParameters.SpineInput
    def __init__(self, from_source: bool = ..., feature_service: Optional[Union[_fco_locator__client_pb2.IdFcoLocator, Mapping]] = ..., feature_view: Optional[Union[_fco_locator__client_pb2.IdFcoLocator, Mapping]] = ..., dataset: Optional[Union[_fco_locator__client_pb2.IdFcoLocator, Mapping]] = ..., dataset_name: Optional[str] = ..., spine: Optional[Union[DatasetGenerationParameters.SpineInput, Mapping]] = ..., datetime_range: Optional[Union[DatasetGenerationParameters.DateTimeRangeInput, Mapping]] = ..., batch_config: Optional[Union[DatasetGenerationParameters.BatchConfigInput, Mapping]] = ..., result_path: Optional[str] = ..., cluster_config: Optional[Union[_feature_view__client_pb2.ClusterConfig, Mapping]] = ..., extra_config: Optional[Mapping[str, str]] = ..., expected_schema: Optional[Union[_schema__client_pb2.Schema, Mapping]] = ...) -> None: ...

class DeletionParameters(_message.Message):
    __slots__ = ["offline", "offline_join_keys_path", "online", "online_join_keys_full_path", "online_join_keys_path"]
    OFFLINE_FIELD_NUMBER: ClassVar[int]
    OFFLINE_JOIN_KEYS_PATH_FIELD_NUMBER: ClassVar[int]
    ONLINE_FIELD_NUMBER: ClassVar[int]
    ONLINE_JOIN_KEYS_FULL_PATH_FIELD_NUMBER: ClassVar[int]
    ONLINE_JOIN_KEYS_PATH_FIELD_NUMBER: ClassVar[int]
    offline: bool
    offline_join_keys_path: str
    online: bool
    online_join_keys_full_path: str
    online_join_keys_path: str
    def __init__(self, online_join_keys_path: Optional[str] = ..., online_join_keys_full_path: Optional[str] = ..., offline_join_keys_path: Optional[str] = ..., online: bool = ..., offline: bool = ...) -> None: ...

class DeltaMaintenanceParameters(_message.Message):
    __slots__ = ["execute_compaction", "execute_sorting", "generate_manifest", "period_end", "vacuum", "vacuum_retention_hours"]
    EXECUTE_COMPACTION_FIELD_NUMBER: ClassVar[int]
    EXECUTE_SORTING_FIELD_NUMBER: ClassVar[int]
    GENERATE_MANIFEST_FIELD_NUMBER: ClassVar[int]
    PERIOD_END_FIELD_NUMBER: ClassVar[int]
    VACUUM_FIELD_NUMBER: ClassVar[int]
    VACUUM_RETENTION_HOURS_FIELD_NUMBER: ClassVar[int]
    execute_compaction: bool
    execute_sorting: bool
    generate_manifest: bool
    period_end: _timestamp_pb2.Timestamp
    vacuum: bool
    vacuum_retention_hours: int
    def __init__(self, generate_manifest: bool = ..., period_end: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., execute_compaction: bool = ..., vacuum: bool = ..., execute_sorting: bool = ..., vacuum_retention_hours: Optional[int] = ...) -> None: ...

class FeatureExportParameters(_message.Message):
    __slots__ = ["export_store_path", "feature_end_time", "feature_start_time", "parent_materialization_task_id"]
    EXPORT_STORE_PATH_FIELD_NUMBER: ClassVar[int]
    FEATURE_END_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: ClassVar[int]
    PARENT_MATERIALIZATION_TASK_ID_FIELD_NUMBER: ClassVar[int]
    export_store_path: str
    feature_end_time: _timestamp_pb2.Timestamp
    feature_start_time: _timestamp_pb2.Timestamp
    parent_materialization_task_id: _id__client_pb2.Id
    def __init__(self, parent_materialization_task_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., feature_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., export_store_path: Optional[str] = ...) -> None: ...

class IngestMaterializationParameters(_message.Message):
    __slots__ = ["ingest_path", "write_to_offline_feature_store", "write_to_online_feature_store"]
    INGEST_PATH_FIELD_NUMBER: ClassVar[int]
    WRITE_TO_OFFLINE_FEATURE_STORE_FIELD_NUMBER: ClassVar[int]
    WRITE_TO_ONLINE_FEATURE_STORE_FIELD_NUMBER: ClassVar[int]
    ingest_path: str
    write_to_offline_feature_store: bool
    write_to_online_feature_store: bool
    def __init__(self, write_to_online_feature_store: bool = ..., write_to_offline_feature_store: bool = ..., ingest_path: Optional[str] = ...) -> None: ...

class MaterializationTask(_message.Message):
    __slots__ = ["attempt_status_message", "attempts", "batch_parameters", "canary_params", "compute_mode", "created_at", "dataset_generation_parameters", "deletion_parameters", "delta_maintenance_parameters", "error_message", "feature_export_parameters", "id_feature_service_locator", "id_feature_view_locator", "ingest_parameters", "managed_retries", "manually_triggered", "materialization_serial_version", "materialization_task_id", "number_of_attempts_from_old_executions", "plan_id", "spark_cluster_environment_version", "state_transitions", "stream_parameters", "tecton_environment", "tecton_runtime_version", "updated_at"]
    ATTEMPTS_FIELD_NUMBER: ClassVar[int]
    ATTEMPT_STATUS_MESSAGE_FIELD_NUMBER: ClassVar[int]
    BATCH_PARAMETERS_FIELD_NUMBER: ClassVar[int]
    CANARY_PARAMS_FIELD_NUMBER: ClassVar[int]
    COMPUTE_MODE_FIELD_NUMBER: ClassVar[int]
    CREATED_AT_FIELD_NUMBER: ClassVar[int]
    DATASET_GENERATION_PARAMETERS_FIELD_NUMBER: ClassVar[int]
    DELETION_PARAMETERS_FIELD_NUMBER: ClassVar[int]
    DELTA_MAINTENANCE_PARAMETERS_FIELD_NUMBER: ClassVar[int]
    ERROR_MESSAGE_FIELD_NUMBER: ClassVar[int]
    FEATURE_EXPORT_PARAMETERS_FIELD_NUMBER: ClassVar[int]
    ID_FEATURE_SERVICE_LOCATOR_FIELD_NUMBER: ClassVar[int]
    ID_FEATURE_VIEW_LOCATOR_FIELD_NUMBER: ClassVar[int]
    INGEST_PARAMETERS_FIELD_NUMBER: ClassVar[int]
    MANAGED_RETRIES_FIELD_NUMBER: ClassVar[int]
    MANUALLY_TRIGGERED_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_SERIAL_VERSION_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_TASK_ID_FIELD_NUMBER: ClassVar[int]
    NUMBER_OF_ATTEMPTS_FROM_OLD_EXECUTIONS_FIELD_NUMBER: ClassVar[int]
    PLAN_ID_FIELD_NUMBER: ClassVar[int]
    SPARK_CLUSTER_ENVIRONMENT_VERSION_FIELD_NUMBER: ClassVar[int]
    STATE_TRANSITIONS_FIELD_NUMBER: ClassVar[int]
    STREAM_PARAMETERS_FIELD_NUMBER: ClassVar[int]
    TECTON_ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    TECTON_RUNTIME_VERSION_FIELD_NUMBER: ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: ClassVar[int]
    attempt_status_message: str
    attempts: _containers.RepeatedCompositeFieldContainer[Attempt]
    batch_parameters: BatchMaterializationParameters
    canary_params: CanaryParams
    compute_mode: _compute_mode__client_pb2.BatchComputeMode
    created_at: _timestamp_pb2.Timestamp
    dataset_generation_parameters: DatasetGenerationParameters
    deletion_parameters: DeletionParameters
    delta_maintenance_parameters: DeltaMaintenanceParameters
    error_message: str
    feature_export_parameters: FeatureExportParameters
    id_feature_service_locator: _fco_locator__client_pb2.IdFcoLocator
    id_feature_view_locator: _fco_locator__client_pb2.IdFcoLocator
    ingest_parameters: IngestMaterializationParameters
    managed_retries: bool
    manually_triggered: bool
    materialization_serial_version: int
    materialization_task_id: _id__client_pb2.Id
    number_of_attempts_from_old_executions: int
    plan_id: _id__client_pb2.Id
    spark_cluster_environment_version: int
    state_transitions: _containers.RepeatedCompositeFieldContainer[MaterializationTaskStateTransition]
    stream_parameters: StreamMaterializationParameters
    tecton_environment: str
    tecton_runtime_version: str
    updated_at: _timestamp_pb2.Timestamp
    def __init__(self, materialization_task_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., plan_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., id_feature_view_locator: Optional[Union[_fco_locator__client_pb2.IdFcoLocator, Mapping]] = ..., id_feature_service_locator: Optional[Union[_fco_locator__client_pb2.IdFcoLocator, Mapping]] = ..., materialization_serial_version: Optional[int] = ..., tecton_runtime_version: Optional[str] = ..., tecton_environment: Optional[str] = ..., compute_mode: Optional[Union[_compute_mode__client_pb2.BatchComputeMode, str]] = ..., batch_parameters: Optional[Union[BatchMaterializationParameters, Mapping]] = ..., stream_parameters: Optional[Union[StreamMaterializationParameters, Mapping]] = ..., ingest_parameters: Optional[Union[IngestMaterializationParameters, Mapping]] = ..., deletion_parameters: Optional[Union[DeletionParameters, Mapping]] = ..., delta_maintenance_parameters: Optional[Union[DeltaMaintenanceParameters, Mapping]] = ..., feature_export_parameters: Optional[Union[FeatureExportParameters, Mapping]] = ..., dataset_generation_parameters: Optional[Union[DatasetGenerationParameters, Mapping]] = ..., spark_cluster_environment_version: Optional[int] = ..., state_transitions: Optional[Iterable[Union[MaterializationTaskStateTransition, Mapping]]] = ..., number_of_attempts_from_old_executions: Optional[int] = ..., error_message: Optional[str] = ..., created_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., updated_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., attempts: Optional[Iterable[Union[Attempt, Mapping]]] = ..., canary_params: Optional[Union[CanaryParams, Mapping]] = ..., manually_triggered: bool = ..., managed_retries: bool = ..., attempt_status_message: Optional[str] = ...) -> None: ...

class MaterializationTaskAttempt(_message.Message):
    __slots__ = ["attempt_number", "cluster_config", "created_at", "execution_environment", "materialization_task_attempt_id", "materialization_task_id", "online_store_copier_tasks", "run_id", "run_page_url", "snowflake_data", "state_transitions", "updated_at"]
    class OnlineStoreCopierTasksEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: OnlineStoreCopierTask
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[OnlineStoreCopierTask, Mapping]] = ...) -> None: ...
    ATTEMPT_NUMBER_FIELD_NUMBER: ClassVar[int]
    CLUSTER_CONFIG_FIELD_NUMBER: ClassVar[int]
    CREATED_AT_FIELD_NUMBER: ClassVar[int]
    EXECUTION_ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_TASK_ATTEMPT_ID_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_TASK_ID_FIELD_NUMBER: ClassVar[int]
    ONLINE_STORE_COPIER_TASKS_FIELD_NUMBER: ClassVar[int]
    RUN_ID_FIELD_NUMBER: ClassVar[int]
    RUN_PAGE_URL_FIELD_NUMBER: ClassVar[int]
    SNOWFLAKE_DATA_FIELD_NUMBER: ClassVar[int]
    STATE_TRANSITIONS_FIELD_NUMBER: ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: ClassVar[int]
    attempt_number: int
    cluster_config: _feature_view__client_pb2.ClusterConfig
    created_at: _timestamp_pb2.Timestamp
    execution_environment: _spark_cluster__client_pb2.SparkExecutionEnvironment
    materialization_task_attempt_id: _id__client_pb2.Id
    materialization_task_id: _id__client_pb2.Id
    online_store_copier_tasks: _containers.MessageMap[str, OnlineStoreCopierTask]
    run_id: str
    run_page_url: str
    snowflake_data: SnowflakeData
    state_transitions: _containers.RepeatedCompositeFieldContainer[MaterializationTaskAttemptStateTransition]
    updated_at: _timestamp_pb2.Timestamp
    def __init__(self, materialization_task_attempt_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., materialization_task_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., execution_environment: Optional[Union[_spark_cluster__client_pb2.SparkExecutionEnvironment, str]] = ..., run_id: Optional[str] = ..., run_page_url: Optional[str] = ..., state_transitions: Optional[Iterable[Union[MaterializationTaskAttemptStateTransition, Mapping]]] = ..., attempt_number: Optional[int] = ..., created_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., updated_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., cluster_config: Optional[Union[_feature_view__client_pb2.ClusterConfig, Mapping]] = ..., online_store_copier_tasks: Optional[Mapping[str, OnlineStoreCopierTask]] = ..., snowflake_data: Optional[Union[SnowflakeData, Mapping]] = ...) -> None: ...

class MaterializationTaskAttemptStateTransition(_message.Message):
    __slots__ = ["attempt_state", "state_message", "termination_reason", "timestamp"]
    ATTEMPT_STATE_FIELD_NUMBER: ClassVar[int]
    STATE_MESSAGE_FIELD_NUMBER: ClassVar[int]
    TERMINATION_REASON_FIELD_NUMBER: ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    attempt_state: _materialization_states__client_pb2.MaterializationTaskAttemptState
    state_message: str
    termination_reason: _jobs__client_pb2.RunTerminationReason
    timestamp: _timestamp_pb2.Timestamp
    def __init__(self, attempt_state: Optional[Union[_materialization_states__client_pb2.MaterializationTaskAttemptState, str]] = ..., state_message: Optional[str] = ..., termination_reason: Optional[Union[_jobs__client_pb2.RunTerminationReason, str]] = ..., timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class MaterializationTaskDep(_message.Message):
    __slots__ = ["offline_store_dependency"]
    OFFLINE_STORE_DEPENDENCY_FIELD_NUMBER: ClassVar[int]
    offline_store_dependency: OfflineStoreDependency
    def __init__(self, offline_store_dependency: Optional[Union[OfflineStoreDependency, Mapping]] = ...) -> None: ...

class MaterializationTaskStateTransition(_message.Message):
    __slots__ = ["task_state", "timestamp"]
    TASK_STATE_FIELD_NUMBER: ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    task_state: _materialization_states__client_pb2.MaterializationTaskState
    timestamp: _timestamp_pb2.Timestamp
    def __init__(self, task_state: Optional[Union[_materialization_states__client_pb2.MaterializationTaskState, str]] = ..., timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class OfflineStoreDependency(_message.Message):
    __slots__ = ["feature_end_time", "feature_start_time"]
    FEATURE_END_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: ClassVar[int]
    feature_end_time: _timestamp_pb2.Timestamp
    feature_start_time: _timestamp_pb2.Timestamp
    def __init__(self, feature_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class OnlineStoreCopierTask(_message.Message):
    __slots__ = ["attempts", "completed_rows", "last_progress_request_id", "last_progress_send_time", "total_rows"]
    ATTEMPTS_FIELD_NUMBER: ClassVar[int]
    COMPLETED_ROWS_FIELD_NUMBER: ClassVar[int]
    LAST_PROGRESS_REQUEST_ID_FIELD_NUMBER: ClassVar[int]
    LAST_PROGRESS_SEND_TIME_FIELD_NUMBER: ClassVar[int]
    TOTAL_ROWS_FIELD_NUMBER: ClassVar[int]
    attempts: _containers.RepeatedCompositeFieldContainer[OnlineStoreCopierTaskAttempt]
    completed_rows: int
    last_progress_request_id: str
    last_progress_send_time: _timestamp_pb2.Timestamp
    total_rows: int
    def __init__(self, attempts: Optional[Iterable[Union[OnlineStoreCopierTaskAttempt, Mapping]]] = ..., total_rows: Optional[int] = ..., completed_rows: Optional[int] = ..., last_progress_request_id: Optional[str] = ..., last_progress_send_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class OnlineStoreCopierTaskAttempt(_message.Message):
    __slots__ = ["state_transitions"]
    STATE_TRANSITIONS_FIELD_NUMBER: ClassVar[int]
    state_transitions: _containers.RepeatedCompositeFieldContainer[OnlineStoreCopierTaskStateTransition]
    def __init__(self, state_transitions: Optional[Iterable[Union[OnlineStoreCopierTaskStateTransition, Mapping]]] = ...) -> None: ...

class OnlineStoreCopierTaskStateTransition(_message.Message):
    __slots__ = ["state", "state_message", "timestamp"]
    STATE_FIELD_NUMBER: ClassVar[int]
    STATE_MESSAGE_FIELD_NUMBER: ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    state: _materialization_states__client_pb2.OnlineStoreCopierTaskState
    state_message: str
    timestamp: _timestamp_pb2.Timestamp
    def __init__(self, state: Optional[Union[_materialization_states__client_pb2.OnlineStoreCopierTaskState, str]] = ..., state_message: Optional[str] = ..., timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class SnowflakeData(_message.Message):
    __slots__ = ["base_location", "total_offline_rows", "total_online_rows"]
    BASE_LOCATION_FIELD_NUMBER: ClassVar[int]
    TOTAL_OFFLINE_ROWS_FIELD_NUMBER: ClassVar[int]
    TOTAL_ONLINE_ROWS_FIELD_NUMBER: ClassVar[int]
    base_location: _location__client_pb2.StageLocation
    total_offline_rows: int
    total_online_rows: int
    def __init__(self, base_location: Optional[Union[_location__client_pb2.StageLocation, Mapping]] = ..., total_online_rows: Optional[int] = ..., total_offline_rows: Optional[int] = ...) -> None: ...

class StreamHandoffConfig(_message.Message):
    __slots__ = ["enabled", "previous_task_id"]
    ENABLED_FIELD_NUMBER: ClassVar[int]
    PREVIOUS_TASK_ID_FIELD_NUMBER: ClassVar[int]
    enabled: bool
    previous_task_id: _id__client_pb2.Id
    def __init__(self, enabled: bool = ..., previous_task_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ...) -> None: ...

class StreamMaterializationParameters(_message.Message):
    __slots__ = ["stream_handoff_config"]
    STREAM_HANDOFF_CONFIG_FIELD_NUMBER: ClassVar[int]
    stream_handoff_config: StreamHandoffConfig
    def __init__(self, stream_handoff_config: Optional[Union[StreamHandoffConfig, Mapping]] = ...) -> None: ...



from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import fco_locator__client_pb2 as _fco_locator__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.dataobs import expectation__client_pb2 as _expectation__client_pb2
from tecton_proto.dataobs import validation__client_pb2 as _validation__client_pb2
from tecton_proto.dataobs import validation_task_params__client_pb2 as _validation_task_params__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor
VALIDATION_TASK_TYPE_BATCH_METRICS: ValidationTaskType
VALIDATION_TASK_TYPE_UNKNOWN: ValidationTaskType

class ValidationTask(_message.Message):
    __slots__ = ["dynamo_data_source", "feature_end_time", "feature_start_time", "feature_view_locator", "metric_expectations", "s3_data_source", "task_type", "timeout", "validation_job_id"]
    DYNAMO_DATA_SOURCE_FIELD_NUMBER: ClassVar[int]
    FEATURE_END_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_LOCATOR_FIELD_NUMBER: ClassVar[int]
    METRIC_EXPECTATIONS_FIELD_NUMBER: ClassVar[int]
    S3_DATA_SOURCE_FIELD_NUMBER: ClassVar[int]
    TASK_TYPE_FIELD_NUMBER: ClassVar[int]
    TIMEOUT_FIELD_NUMBER: ClassVar[int]
    VALIDATION_JOB_ID_FIELD_NUMBER: ClassVar[int]
    dynamo_data_source: _validation_task_params__client_pb2.DynamoDataSource
    feature_end_time: _timestamp_pb2.Timestamp
    feature_start_time: _timestamp_pb2.Timestamp
    feature_view_locator: _fco_locator__client_pb2.IdFcoLocator
    metric_expectations: _containers.RepeatedCompositeFieldContainer[_expectation__client_pb2.MetricExpectation]
    s3_data_source: _validation_task_params__client_pb2.S3DataSource
    task_type: ValidationTaskType
    timeout: _duration_pb2.Duration
    validation_job_id: _id__client_pb2.Id
    def __init__(self, validation_job_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., feature_view_locator: Optional[Union[_fco_locator__client_pb2.IdFcoLocator, Mapping]] = ..., metric_expectations: Optional[Iterable[Union[_expectation__client_pb2.MetricExpectation, Mapping]]] = ..., feature_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., task_type: Optional[Union[ValidationTaskType, str]] = ..., timeout: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., dynamo_data_source: Optional[Union[_validation_task_params__client_pb2.DynamoDataSource, Mapping]] = ..., s3_data_source: Optional[Union[_validation_task_params__client_pb2.S3DataSource, Mapping]] = ...) -> None: ...

class ValidationTaskMetrics(_message.Message):
    __slots__ = ["feature_rows_read", "metric_rows_read", "query_execution_times"]
    FEATURE_ROWS_READ_FIELD_NUMBER: ClassVar[int]
    METRIC_ROWS_READ_FIELD_NUMBER: ClassVar[int]
    QUERY_EXECUTION_TIMES_FIELD_NUMBER: ClassVar[int]
    feature_rows_read: int
    metric_rows_read: int
    query_execution_times: _containers.RepeatedCompositeFieldContainer[_duration_pb2.Duration]
    def __init__(self, metric_rows_read: Optional[int] = ..., feature_rows_read: Optional[int] = ..., query_execution_times: Optional[Iterable[Union[_duration_pb2.Duration, Mapping]]] = ...) -> None: ...

class ValidationTaskResult(_message.Message):
    __slots__ = ["feature_end_time", "feature_package_id", "feature_start_time", "metrics", "results", "validation_job_id", "validation_time", "workspace"]
    FEATURE_END_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_PACKAGE_ID_FIELD_NUMBER: ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: ClassVar[int]
    METRICS_FIELD_NUMBER: ClassVar[int]
    RESULTS_FIELD_NUMBER: ClassVar[int]
    VALIDATION_JOB_ID_FIELD_NUMBER: ClassVar[int]
    VALIDATION_TIME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    feature_end_time: _timestamp_pb2.Timestamp
    feature_package_id: _id__client_pb2.Id
    feature_start_time: _timestamp_pb2.Timestamp
    metrics: ValidationTaskMetrics
    results: _containers.RepeatedCompositeFieldContainer[_validation__client_pb2.ExpectationResult]
    validation_job_id: _id__client_pb2.Id
    validation_time: _timestamp_pb2.Timestamp
    workspace: str
    def __init__(self, workspace: Optional[str] = ..., feature_package_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., validation_job_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., feature_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., results: Optional[Iterable[Union[_validation__client_pb2.ExpectationResult, Mapping]]] = ..., metrics: Optional[Union[ValidationTaskMetrics, Mapping]] = ..., validation_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class ValidationTaskType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor
HTTP: Scheme
HTTPS: Scheme
UNKNOWN: Scheme
WS: Scheme
WSS: Scheme

class Contact(_message.Message):
    __slots__ = ["email", "name", "url"]
    EMAIL_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    URL_FIELD_NUMBER: ClassVar[int]
    email: str
    name: str
    url: str
    def __init__(self, name: Optional[str] = ..., url: Optional[str] = ..., email: Optional[str] = ...) -> None: ...

class ExternalDocumentation(_message.Message):
    __slots__ = ["description", "url"]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    URL_FIELD_NUMBER: ClassVar[int]
    description: str
    url: str
    def __init__(self, description: Optional[str] = ..., url: Optional[str] = ...) -> None: ...

class Header(_message.Message):
    __slots__ = ["default", "description", "format", "pattern", "type"]
    DEFAULT_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    FORMAT_FIELD_NUMBER: ClassVar[int]
    PATTERN_FIELD_NUMBER: ClassVar[int]
    TYPE_FIELD_NUMBER: ClassVar[int]
    default: str
    description: str
    format: str
    pattern: str
    type: str
    def __init__(self, description: Optional[str] = ..., type: Optional[str] = ..., format: Optional[str] = ..., default: Optional[str] = ..., pattern: Optional[str] = ...) -> None: ...

class HeaderParameter(_message.Message):
    __slots__ = ["description", "format", "name", "required", "type"]
    class Type(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    BOOLEAN: HeaderParameter.Type
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    FORMAT_FIELD_NUMBER: ClassVar[int]
    INTEGER: HeaderParameter.Type
    NAME_FIELD_NUMBER: ClassVar[int]
    NUMBER: HeaderParameter.Type
    REQUIRED_FIELD_NUMBER: ClassVar[int]
    STRING: HeaderParameter.Type
    TYPE_FIELD_NUMBER: ClassVar[int]
    UNKNOWN: HeaderParameter.Type
    description: str
    format: str
    name: str
    required: bool
    type: HeaderParameter.Type
    def __init__(self, name: Optional[str] = ..., description: Optional[str] = ..., type: Optional[Union[HeaderParameter.Type, str]] = ..., format: Optional[str] = ..., required: bool = ...) -> None: ...

class Info(_message.Message):
    __slots__ = ["contact", "description", "extensions", "license", "terms_of_service", "title", "version"]
    class ExtensionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    CONTACT_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    EXTENSIONS_FIELD_NUMBER: ClassVar[int]
    LICENSE_FIELD_NUMBER: ClassVar[int]
    TERMS_OF_SERVICE_FIELD_NUMBER: ClassVar[int]
    TITLE_FIELD_NUMBER: ClassVar[int]
    VERSION_FIELD_NUMBER: ClassVar[int]
    contact: Contact
    description: str
    extensions: _containers.MessageMap[str, _struct_pb2.Value]
    license: License
    terms_of_service: str
    title: str
    version: str
    def __init__(self, title: Optional[str] = ..., description: Optional[str] = ..., terms_of_service: Optional[str] = ..., contact: Optional[Union[Contact, Mapping]] = ..., license: Optional[Union[License, Mapping]] = ..., version: Optional[str] = ..., extensions: Optional[Mapping[str, _struct_pb2.Value]] = ...) -> None: ...

class JSONSchema(_message.Message):
    __slots__ = ["array", "default", "description", "enum", "example", "exclusive_maximum", "exclusive_minimum", "extensions", "field_configuration", "format", "max_items", "max_length", "max_properties", "maximum", "min_items", "min_length", "min_properties", "minimum", "multiple_of", "pattern", "read_only", "ref", "required", "title", "type", "unique_items"]
    class JSONSchemaSimpleTypes(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class ExtensionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    class FieldConfiguration(_message.Message):
        __slots__ = ["path_param_name"]
        PATH_PARAM_NAME_FIELD_NUMBER: ClassVar[int]
        path_param_name: str
        def __init__(self, path_param_name: Optional[str] = ...) -> None: ...
    ARRAY: JSONSchema.JSONSchemaSimpleTypes
    ARRAY_FIELD_NUMBER: ClassVar[int]
    BOOLEAN: JSONSchema.JSONSchemaSimpleTypes
    DEFAULT_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    ENUM_FIELD_NUMBER: ClassVar[int]
    EXAMPLE_FIELD_NUMBER: ClassVar[int]
    EXCLUSIVE_MAXIMUM_FIELD_NUMBER: ClassVar[int]
    EXCLUSIVE_MINIMUM_FIELD_NUMBER: ClassVar[int]
    EXTENSIONS_FIELD_NUMBER: ClassVar[int]
    FIELD_CONFIGURATION_FIELD_NUMBER: ClassVar[int]
    FORMAT_FIELD_NUMBER: ClassVar[int]
    INTEGER: JSONSchema.JSONSchemaSimpleTypes
    MAXIMUM_FIELD_NUMBER: ClassVar[int]
    MAX_ITEMS_FIELD_NUMBER: ClassVar[int]
    MAX_LENGTH_FIELD_NUMBER: ClassVar[int]
    MAX_PROPERTIES_FIELD_NUMBER: ClassVar[int]
    MINIMUM_FIELD_NUMBER: ClassVar[int]
    MIN_ITEMS_FIELD_NUMBER: ClassVar[int]
    MIN_LENGTH_FIELD_NUMBER: ClassVar[int]
    MIN_PROPERTIES_FIELD_NUMBER: ClassVar[int]
    MULTIPLE_OF_FIELD_NUMBER: ClassVar[int]
    NULL: JSONSchema.JSONSchemaSimpleTypes
    NUMBER: JSONSchema.JSONSchemaSimpleTypes
    OBJECT: JSONSchema.JSONSchemaSimpleTypes
    PATTERN_FIELD_NUMBER: ClassVar[int]
    READ_ONLY_FIELD_NUMBER: ClassVar[int]
    REF_FIELD_NUMBER: ClassVar[int]
    REQUIRED_FIELD_NUMBER: ClassVar[int]
    STRING: JSONSchema.JSONSchemaSimpleTypes
    TITLE_FIELD_NUMBER: ClassVar[int]
    TYPE_FIELD_NUMBER: ClassVar[int]
    UNIQUE_ITEMS_FIELD_NUMBER: ClassVar[int]
    UNKNOWN: JSONSchema.JSONSchemaSimpleTypes
    array: _containers.RepeatedScalarFieldContainer[str]
    default: str
    description: str
    enum: _containers.RepeatedScalarFieldContainer[str]
    example: str
    exclusive_maximum: bool
    exclusive_minimum: bool
    extensions: _containers.MessageMap[str, _struct_pb2.Value]
    field_configuration: JSONSchema.FieldConfiguration
    format: str
    max_items: int
    max_length: int
    max_properties: int
    maximum: float
    min_items: int
    min_length: int
    min_properties: int
    minimum: float
    multiple_of: float
    pattern: str
    read_only: bool
    ref: str
    required: _containers.RepeatedScalarFieldContainer[str]
    title: str
    type: _containers.RepeatedScalarFieldContainer[JSONSchema.JSONSchemaSimpleTypes]
    unique_items: bool
    def __init__(self, ref: Optional[str] = ..., title: Optional[str] = ..., description: Optional[str] = ..., default: Optional[str] = ..., read_only: bool = ..., example: Optional[str] = ..., multiple_of: Optional[float] = ..., maximum: Optional[float] = ..., exclusive_maximum: bool = ..., minimum: Optional[float] = ..., exclusive_minimum: bool = ..., max_length: Optional[int] = ..., min_length: Optional[int] = ..., pattern: Optional[str] = ..., max_items: Optional[int] = ..., min_items: Optional[int] = ..., unique_items: bool = ..., max_properties: Optional[int] = ..., min_properties: Optional[int] = ..., required: Optional[Iterable[str]] = ..., array: Optional[Iterable[str]] = ..., type: Optional[Iterable[Union[JSONSchema.JSONSchemaSimpleTypes, str]]] = ..., format: Optional[str] = ..., enum: Optional[Iterable[str]] = ..., field_configuration: Optional[Union[JSONSchema.FieldConfiguration, Mapping]] = ..., extensions: Optional[Mapping[str, _struct_pb2.Value]] = ...) -> None: ...

class License(_message.Message):
    __slots__ = ["name", "url"]
    NAME_FIELD_NUMBER: ClassVar[int]
    URL_FIELD_NUMBER: ClassVar[int]
    name: str
    url: str
    def __init__(self, name: Optional[str] = ..., url: Optional[str] = ...) -> None: ...

class Operation(_message.Message):
    __slots__ = ["consumes", "deprecated", "description", "extensions", "external_docs", "operation_id", "parameters", "produces", "responses", "schemes", "security", "summary", "tags"]
    class ExtensionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    class ResponsesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: Response
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[Response, Mapping]] = ...) -> None: ...
    CONSUMES_FIELD_NUMBER: ClassVar[int]
    DEPRECATED_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    EXTENSIONS_FIELD_NUMBER: ClassVar[int]
    EXTERNAL_DOCS_FIELD_NUMBER: ClassVar[int]
    OPERATION_ID_FIELD_NUMBER: ClassVar[int]
    PARAMETERS_FIELD_NUMBER: ClassVar[int]
    PRODUCES_FIELD_NUMBER: ClassVar[int]
    RESPONSES_FIELD_NUMBER: ClassVar[int]
    SCHEMES_FIELD_NUMBER: ClassVar[int]
    SECURITY_FIELD_NUMBER: ClassVar[int]
    SUMMARY_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    consumes: _containers.RepeatedScalarFieldContainer[str]
    deprecated: bool
    description: str
    extensions: _containers.MessageMap[str, _struct_pb2.Value]
    external_docs: ExternalDocumentation
    operation_id: str
    parameters: Parameters
    produces: _containers.RepeatedScalarFieldContainer[str]
    responses: _containers.MessageMap[str, Response]
    schemes: _containers.RepeatedScalarFieldContainer[Scheme]
    security: _containers.RepeatedCompositeFieldContainer[SecurityRequirement]
    summary: str
    tags: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, tags: Optional[Iterable[str]] = ..., summary: Optional[str] = ..., description: Optional[str] = ..., external_docs: Optional[Union[ExternalDocumentation, Mapping]] = ..., operation_id: Optional[str] = ..., consumes: Optional[Iterable[str]] = ..., produces: Optional[Iterable[str]] = ..., responses: Optional[Mapping[str, Response]] = ..., schemes: Optional[Iterable[Union[Scheme, str]]] = ..., deprecated: bool = ..., security: Optional[Iterable[Union[SecurityRequirement, Mapping]]] = ..., extensions: Optional[Mapping[str, _struct_pb2.Value]] = ..., parameters: Optional[Union[Parameters, Mapping]] = ...) -> None: ...

class Parameters(_message.Message):
    __slots__ = ["headers"]
    HEADERS_FIELD_NUMBER: ClassVar[int]
    headers: _containers.RepeatedCompositeFieldContainer[HeaderParameter]
    def __init__(self, headers: Optional[Iterable[Union[HeaderParameter, Mapping]]] = ...) -> None: ...

class Response(_message.Message):
    __slots__ = ["description", "examples", "extensions", "headers", "schema"]
    class ExamplesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    class ExtensionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    class HeadersEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: Header
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[Header, Mapping]] = ...) -> None: ...
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    EXAMPLES_FIELD_NUMBER: ClassVar[int]
    EXTENSIONS_FIELD_NUMBER: ClassVar[int]
    HEADERS_FIELD_NUMBER: ClassVar[int]
    SCHEMA_FIELD_NUMBER: ClassVar[int]
    description: str
    examples: _containers.ScalarMap[str, str]
    extensions: _containers.MessageMap[str, _struct_pb2.Value]
    headers: _containers.MessageMap[str, Header]
    schema: Schema
    def __init__(self, description: Optional[str] = ..., schema: Optional[Union[Schema, Mapping]] = ..., headers: Optional[Mapping[str, Header]] = ..., examples: Optional[Mapping[str, str]] = ..., extensions: Optional[Mapping[str, _struct_pb2.Value]] = ...) -> None: ...

class Schema(_message.Message):
    __slots__ = ["discriminator", "example", "external_docs", "json_schema", "read_only"]
    DISCRIMINATOR_FIELD_NUMBER: ClassVar[int]
    EXAMPLE_FIELD_NUMBER: ClassVar[int]
    EXTERNAL_DOCS_FIELD_NUMBER: ClassVar[int]
    JSON_SCHEMA_FIELD_NUMBER: ClassVar[int]
    READ_ONLY_FIELD_NUMBER: ClassVar[int]
    discriminator: str
    example: str
    external_docs: ExternalDocumentation
    json_schema: JSONSchema
    read_only: bool
    def __init__(self, json_schema: Optional[Union[JSONSchema, Mapping]] = ..., discriminator: Optional[str] = ..., read_only: bool = ..., external_docs: Optional[Union[ExternalDocumentation, Mapping]] = ..., example: Optional[str] = ...) -> None: ...

class Scopes(_message.Message):
    __slots__ = ["scope"]
    class ScopeEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    SCOPE_FIELD_NUMBER: ClassVar[int]
    scope: _containers.ScalarMap[str, str]
    def __init__(self, scope: Optional[Mapping[str, str]] = ...) -> None: ...

class SecurityDefinitions(_message.Message):
    __slots__ = ["security"]
    class SecurityEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: SecurityScheme
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[SecurityScheme, Mapping]] = ...) -> None: ...
    SECURITY_FIELD_NUMBER: ClassVar[int]
    security: _containers.MessageMap[str, SecurityScheme]
    def __init__(self, security: Optional[Mapping[str, SecurityScheme]] = ...) -> None: ...

class SecurityRequirement(_message.Message):
    __slots__ = ["security_requirement"]
    class SecurityRequirementEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: SecurityRequirement.SecurityRequirementValue
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[SecurityRequirement.SecurityRequirementValue, Mapping]] = ...) -> None: ...
    class SecurityRequirementValue(_message.Message):
        __slots__ = ["scope"]
        SCOPE_FIELD_NUMBER: ClassVar[int]
        scope: _containers.RepeatedScalarFieldContainer[str]
        def __init__(self, scope: Optional[Iterable[str]] = ...) -> None: ...
    SECURITY_REQUIREMENT_FIELD_NUMBER: ClassVar[int]
    security_requirement: _containers.MessageMap[str, SecurityRequirement.SecurityRequirementValue]
    def __init__(self, security_requirement: Optional[Mapping[str, SecurityRequirement.SecurityRequirementValue]] = ...) -> None: ...

class SecurityScheme(_message.Message):
    __slots__ = ["authorization_url", "description", "extensions", "flow", "name", "scopes", "token_url", "type"]
    class Flow(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class In(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class Type(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class ExtensionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    AUTHORIZATION_URL_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    EXTENSIONS_FIELD_NUMBER: ClassVar[int]
    FLOW_ACCESS_CODE: SecurityScheme.Flow
    FLOW_APPLICATION: SecurityScheme.Flow
    FLOW_FIELD_NUMBER: ClassVar[int]
    FLOW_IMPLICIT: SecurityScheme.Flow
    FLOW_INVALID: SecurityScheme.Flow
    FLOW_PASSWORD: SecurityScheme.Flow
    IN_FIELD_NUMBER: ClassVar[int]
    IN_HEADER: SecurityScheme.In
    IN_INVALID: SecurityScheme.In
    IN_QUERY: SecurityScheme.In
    NAME_FIELD_NUMBER: ClassVar[int]
    SCOPES_FIELD_NUMBER: ClassVar[int]
    TOKEN_URL_FIELD_NUMBER: ClassVar[int]
    TYPE_API_KEY: SecurityScheme.Type
    TYPE_BASIC: SecurityScheme.Type
    TYPE_FIELD_NUMBER: ClassVar[int]
    TYPE_INVALID: SecurityScheme.Type
    TYPE_OAUTH2: SecurityScheme.Type
    authorization_url: str
    description: str
    extensions: _containers.MessageMap[str, _struct_pb2.Value]
    flow: SecurityScheme.Flow
    name: str
    scopes: Scopes
    token_url: str
    type: SecurityScheme.Type
    def __init__(self, type: Optional[Union[SecurityScheme.Type, str]] = ..., description: Optional[str] = ..., name: Optional[str] = ..., flow: Optional[Union[SecurityScheme.Flow, str]] = ..., authorization_url: Optional[str] = ..., token_url: Optional[str] = ..., scopes: Optional[Union[Scopes, Mapping]] = ..., extensions: Optional[Mapping[str, _struct_pb2.Value]] = ..., **kwargs) -> None: ...

class Swagger(_message.Message):
    __slots__ = ["base_path", "consumes", "extensions", "external_docs", "host", "info", "produces", "responses", "schemes", "security", "security_definitions", "swagger", "tags"]
    class ExtensionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    class ResponsesEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: Response
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[Response, Mapping]] = ...) -> None: ...
    BASE_PATH_FIELD_NUMBER: ClassVar[int]
    CONSUMES_FIELD_NUMBER: ClassVar[int]
    EXTENSIONS_FIELD_NUMBER: ClassVar[int]
    EXTERNAL_DOCS_FIELD_NUMBER: ClassVar[int]
    HOST_FIELD_NUMBER: ClassVar[int]
    INFO_FIELD_NUMBER: ClassVar[int]
    PRODUCES_FIELD_NUMBER: ClassVar[int]
    RESPONSES_FIELD_NUMBER: ClassVar[int]
    SCHEMES_FIELD_NUMBER: ClassVar[int]
    SECURITY_DEFINITIONS_FIELD_NUMBER: ClassVar[int]
    SECURITY_FIELD_NUMBER: ClassVar[int]
    SWAGGER_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    base_path: str
    consumes: _containers.RepeatedScalarFieldContainer[str]
    extensions: _containers.MessageMap[str, _struct_pb2.Value]
    external_docs: ExternalDocumentation
    host: str
    info: Info
    produces: _containers.RepeatedScalarFieldContainer[str]
    responses: _containers.MessageMap[str, Response]
    schemes: _containers.RepeatedScalarFieldContainer[Scheme]
    security: _containers.RepeatedCompositeFieldContainer[SecurityRequirement]
    security_definitions: SecurityDefinitions
    swagger: str
    tags: _containers.RepeatedCompositeFieldContainer[Tag]
    def __init__(self, swagger: Optional[str] = ..., info: Optional[Union[Info, Mapping]] = ..., host: Optional[str] = ..., base_path: Optional[str] = ..., schemes: Optional[Iterable[Union[Scheme, str]]] = ..., consumes: Optional[Iterable[str]] = ..., produces: Optional[Iterable[str]] = ..., responses: Optional[Mapping[str, Response]] = ..., security_definitions: Optional[Union[SecurityDefinitions, Mapping]] = ..., security: Optional[Iterable[Union[SecurityRequirement, Mapping]]] = ..., tags: Optional[Iterable[Union[Tag, Mapping]]] = ..., external_docs: Optional[Union[ExternalDocumentation, Mapping]] = ..., extensions: Optional[Mapping[str, _struct_pb2.Value]] = ...) -> None: ...

class Tag(_message.Message):
    __slots__ = ["description", "extensions", "external_docs", "name"]
    class ExtensionsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    EXTENSIONS_FIELD_NUMBER: ClassVar[int]
    EXTERNAL_DOCS_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    description: str
    extensions: _containers.MessageMap[str, _struct_pb2.Value]
    external_docs: ExternalDocumentation
    name: str
    def __init__(self, name: Optional[str] = ..., description: Optional[str] = ..., external_docs: Optional[Union[ExternalDocumentation, Mapping]] = ..., extensions: Optional[Mapping[str, _struct_pb2.Value]] = ...) -> None: ...

class Scheme(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.auditlog import metadata__client_pb2 as _metadata__client_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class Workspace(_message.Message):
    __slots__ = ["capabilities", "created_at", "name"]
    CAPABILITIES_FIELD_NUMBER: ClassVar[int]
    CREATED_AT_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    capabilities: WorkspaceCapabilities
    created_at: _timestamp_pb2.Timestamp
    name: str
    def __init__(self, name: Optional[str] = ..., capabilities: Optional[Union[WorkspaceCapabilities, Mapping]] = ..., created_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class WorkspaceCapabilities(_message.Message):
    __slots__ = ["materializable", "offline_store_subdirectory_enabled"]
    MATERIALIZABLE_FIELD_NUMBER: ClassVar[int]
    OFFLINE_STORE_SUBDIRECTORY_ENABLED_FIELD_NUMBER: ClassVar[int]
    materializable: bool
    offline_store_subdirectory_enabled: bool
    def __init__(self, materializable: bool = ..., offline_store_subdirectory_enabled: bool = ...) -> None: ...



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar

DESCRIPTOR: _descriptor.FileDescriptor
FEATURE_STORE_FORMAT_VERSION_DEFAULT: FeatureStoreFormatVersion
FEATURE_STORE_FORMAT_VERSION_MAX: FeatureStoreFormatVersion
FEATURE_STORE_FORMAT_VERSION_ONLINE_STORE_TTL_DELETION_ENABLED: FeatureStoreFormatVersion
FEATURE_STORE_FORMAT_VERSION_TIME_NANOSECONDS: FeatureStoreFormatVersion
FEATURE_STORE_FORMAT_VERSION_TTL_FIELD: FeatureStoreFormatVersion

class FeatureStoreFormatVersion(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from unittest import TestCase

import pandas

from tecton_core import conf
from tecton_core.data_processing_utils import split_spine


df = pandas.DataFrame({"join_key": ["abc", "abd", "dbc", "dbe", "ace", "xyz", "xya", "xzz"]})
join_keys = ["join_key"]


class SpineSplitTest(TestCase):
    def test_spine_split_even(self):
        conf.set("DUCKDB_SPINE_SPLIT_STRATEGY", "even")
        conf.set("DUCKDB_SPINE_SPLIT_COUNT", 3)

        split_dfs = split_spine(df, join_keys)

        assert pandas.DataFrame.equals(split_dfs[0], pandas.DataFrame({"join_key": ["abc", "abd", "ace"]}))
        assert pandas.DataFrame.equals(split_dfs[1], pandas.DataFrame({"join_key": ["dbc", "dbe", "xya"]}))
        assert pandas.DataFrame.equals(split_dfs[2], pandas.DataFrame({"join_key": ["xyz", "xzz"]}))

    def test_spine_split_minimize_distance(self):
        conf.set("DUCKDB_SPINE_SPLIT_STRATEGY", "minimize_distance")
        conf.set("DUCKDB_SPINE_SPLIT_COUNT", 3)

        split_dfs = split_spine(df, join_keys)

        assert pandas.DataFrame.equals(split_dfs[0], pandas.DataFrame({"join_key": ["abc", "abd", "ace"]}))
        assert pandas.DataFrame.equals(split_dfs[1], pandas.DataFrame({"join_key": ["dbc", "dbe"]}))
        assert pandas.DataFrame.equals(split_dfs[2], pandas.DataFrame({"join_key": ["xya", "xyz", "xzz"]}))

    def test_spine_split_agglomerative_clustering(self):
        conf.set("DUCKDB_SPINE_SPLIT_STRATEGY", "agglomerative_clustering")
        conf.set("DUCKDB_SPINE_SPLIT_COUNT", 3)

        split_dfs = split_spine(df, join_keys)
        assert pandas.DataFrame.equals(split_dfs[0], pandas.DataFrame({"join_key": ["xyz", "xya", "xzz"]}))
        assert pandas.DataFrame.equals(split_dfs[1], pandas.DataFrame({"join_key": ["abc", "abd", "ace"]}))
        assert pandas.DataFrame.equals(split_dfs[2], pandas.DataFrame({"join_key": ["dbc", "dbe"]}))



import heapq
from datetime import datetime
from datetime import timedelta
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Union

import numpy
import pandas

from tecton_core import conf
from tecton_core.errors import TectonValidationError
from tecton_core.feature_definition_wrapper import FeatureDefinitionWrapper
from tecton_core.query.dialect import Dialect
from tecton_core.query.nodes import MergeValidityPeriodsNode
from tecton_core.query.nodes import PandasDataframeWrapper
from tecton_core.query.nodes import TrimValidityPeriodNode
from tecton_core.query.nodes import UserSpecifiedDataNode
from tecton_proto.args.pipeline__client_pb2 import PipelineNode


def get_num_dependent_fv(node: PipelineNode, visited_inputs: Dict[str, bool]) -> int:
    if node.HasField("feature_view_node"):
        if node.feature_view_node.input_name in visited_inputs:
            return 0
        visited_inputs[node.feature_view_node.input_name] = True
        return 1
    elif node.HasField("transformation_node"):
        ret = 0
        for child in node.transformation_node.inputs:
            ret = ret + get_num_dependent_fv(child.node, visited_inputs)
        return ret
    return 0


def should_infer_timestamp_of_spine(
    feature_definition: FeatureDefinitionWrapper,
    timestamp_key: Optional[str],
) -> bool:
    if feature_definition.is_rtfv_or_prompt:
        # We need to infer the spine timestamp for a RTFV if the user doesn't pass an explicit timestamp key and
        # 1. It has a dependent materialized feature view or
        # 2. It uses a realtime context
        return timestamp_key is None and (
            get_num_dependent_fv(feature_definition.pipeline.root, visited_inputs={}) > 0
            or feature_definition.uses_realtime_context
        )
    else:
        return timestamp_key is None


def infer_pandas_timestamp(spine: pandas.DataFrame) -> Optional[str]:
    dtypes = dict(spine.dtypes)

    if isinstance(spine, pandas.DataFrame):
        timestamp_cols = [(k, v) for (k, v) in dtypes.items() if pandas.api.types.is_datetime64_any_dtype(v)]
    else:
        msg = f"Unexpected data type for spine: {type(spine)}"
        raise TectonValidationError(msg)

    if len(timestamp_cols) > 1 or len(timestamp_cols) == 0:
        msg = f"Could not infer timestamp keys from {dtypes}; please specify explicitly"
        raise TectonValidationError(msg)
    return timestamp_cols[0][0]


def split_spine(spine: pandas.DataFrame, join_keys: List[str]) -> List[pandas.DataFrame]:
    """
    This is an experimental feature that splits the spine DataFrame into multiple DataFrames based on the spine split
    strategy specified in the configuration. The split strategy can be one of the following:
    - even: Splits the spine into equal-sized chunks
    - minimize_distance: Splits the spine at points where the distance between consecutive keys is maximized
    - agglomerative_clustering: Splits the spine using Agglomerative Clustering based on the bitwise distance between keys
    """
    split_count = int(conf.get_or_none("DUCKDB_SPINE_SPLIT_COUNT"))
    strategy = conf.get_or_none("DUCKDB_SPINE_SPLIT_STRATEGY")

    if strategy == "even":
        return _even_split(spine, join_keys, split_count)
    elif strategy == "minimize_distance":
        return _minimize_distance_split(spine, join_keys, split_count)
    elif strategy == "agglomerative_clustering":
        return _agglomerative_clustering_split(spine, join_keys, split_count)
    else:
        error = f"Unknown spine split strategy: {strategy}"
        raise ValueError(error)


def split_range(range_start: datetime, range_end: datetime, interval: timedelta) -> List[Tuple[datetime, datetime]]:
    """
    Splits a time range into multiple smaller ranges where each split time is aligned to the interval.
    The start of the first range and the end of the last range are preserved.
    """
    split_count = int(conf.get_or_none("DUCKDB_RANGE_SPLIT_COUNT"))

    total_seconds = (range_end - range_start).total_seconds()
    interval_seconds = interval.total_seconds()
    split_seconds = total_seconds / split_count

    split_ranges = []
    current_time = range_start

    for i in range(split_count):
        # For the last interval, ensure the end time matches the provided range_end
        if i == split_count - 1:
            aligned_time = range_end
        else:
            # Calculate the next time and align it
            next_time = current_time + timedelta(seconds=split_seconds)
            epoch_seconds = next_time.timestamp()
            aligned_seconds = (epoch_seconds // interval_seconds) * interval_seconds
            aligned_time = datetime.fromtimestamp(aligned_seconds)

            # Ensure the aligned time does not exceed the range_end
            if aligned_time > range_end:
                aligned_time = range_end

        # Append the current range
        split_ranges.append((current_time, aligned_time))
        current_time = aligned_time

    return split_ranges


def _key_to_int(s: Union[str, int]) -> int:
    if isinstance(s, int):
        return s
    # Convert a string to an integer by concatenating the ASCII values of the characters
    # TODO: This may break when the string is too long
    result = 0
    for i, char in enumerate(s):
        result += ord(char) << (8 * (len(s) - i - 1))
    return result


def _even_split(spine: pandas.DataFrame, join_keys: List[str], split_count: int) -> List[pandas.DataFrame]:
    spine_sorted = spine.sort_values(by=join_keys).reset_index(drop=True)

    # Calculate the number of rows per split
    total_rows = len(spine_sorted)
    rows_per_split = total_rows // split_count
    extra_rows = total_rows % split_count

    # Calculate the split indices
    split_indices = []
    start_idx = 0
    for i in range(split_count):
        end_idx = start_idx + rows_per_split + (1 if i < extra_rows else 0)
        split_indices.append((start_idx, end_idx))
        start_idx = end_idx

    # Split the DataFrame into smaller DataFrames
    split_dfs = [spine_sorted.iloc[start:end].reset_index(drop=True) for start, end in split_indices]

    return split_dfs


def _minimize_distance_split(spine: pandas.DataFrame, join_keys: List[str], split_count: int) -> List[pandas.DataFrame]:
    # Check join_keys type, we only support str and int now
    if not all(
        pandas.api.types.is_integer_dtype(spine[join_key]) or pandas.api.types.is_string_dtype(spine[join_key])
        for join_key in join_keys
    ):
        error = "The join keys must be of type int or str for the minimize_distance spine split strategy."
        raise ValueError(error)

    # Sort the dataframe by the join_key
    spine_sorted = spine.sort_values(by=join_keys).reset_index(drop=True)

    # Compute distances between consecutive keys
    keys = spine_sorted[join_keys[0]].tolist()

    int_keys = [_key_to_int(key) for key in keys]
    distances = [abs(int_keys[i + 1] - int_keys[i]) for i in range(len(int_keys) - 1)]

    # Determine split points using heapq for efficiency with small split_count
    if split_count > 1 and distances:
        largest_indices = heapq.nlargest(split_count - 1, range(len(distances)), distances.__getitem__)
        split_points = sorted([index + 1 for index in largest_indices])
    else:
        split_points = []

    # Split the dataframe into chunks
    start_idx = 0
    split_dfs = []

    for idx in split_points:
        split_dfs.append(spine_sorted.iloc[start_idx:idx].reset_index(drop=True))
        start_idx = idx

    split_dfs.append(spine_sorted.iloc[start_idx:].reset_index(drop=True))  # Add the last chunk

    return split_dfs


def _agglomerative_clustering_split(
    spine: pandas.DataFrame, join_keys: List[str], split_count: int
) -> List[pandas.DataFrame]:
    # Check join_keys type, we only support str and int now
    if not all(
        pandas.api.types.is_integer_dtype(spine[join_key]) or pandas.api.types.is_string_dtype(spine[join_key])
        for join_key in join_keys
    ):
        error = "The join keys must be of type int or str for the agglomerative_clustering spine split strategy."
        raise ValueError(error)
    try:
        from sklearn.cluster import AgglomerativeClustering
    except ImportError:
        error = "Please install the scikit-learn package to use the agglomerative_clustering spine split strategy."
        raise ImportError(error)

    # Function to compute pairwise distances using the bitwise approach
    def bitwise_distance_matrix(keys: List[str]) -> numpy.ndarray:
        int_keys = numpy.array([_key_to_int(key) for key in keys])
        dist_matrix = numpy.abs(int_keys[:, None] - int_keys)
        return dist_matrix

    # Compute the distance matrix using the custom weighted distance function
    keys = spine[join_keys[0]].tolist()
    dist_matrix = bitwise_distance_matrix(keys)

    # Apply Agglomerative Clustering using the precomputed distance matrix
    clustering = AgglomerativeClustering(n_clusters=split_count, metric="precomputed", linkage="complete")
    spine["cluster"] = clustering.fit_predict(dist_matrix)

    # Split the DataFrame into multiple DataFrames based on the clusters
    grouped = spine.groupby("cluster")
    split_dfs = [group.drop(columns="cluster").reset_index(drop=True) for _, group in grouped]

    spine.drop(columns="cluster", inplace=True)

    return split_dfs


def merge_validity_periods(df_list, valid_time_range, fdw):
    df = pandas.concat(df_list)
    qt = UserSpecifiedDataNode(
        dialect=Dialect.DUCKDB,
        compute_mode="rift",
        data=PandasDataframeWrapper(df),
    ).as_ref()

    qt = MergeValidityPeriodsNode(
        dialect=Dialect.DUCKDB,
        compute_mode="rift",
        input_node=qt,
        fdw=fdw,
    ).as_ref()

    qt = TrimValidityPeriodNode(
        dialect=Dialect.DUCKDB,
        compute_mode="rift",
        input_node=qt,
        start=valid_time_range.start,
        end=valid_time_range.end,
    ).as_ref()

    return qt



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.auth import principal__client_pb2 as _principal__client_pb2
from tecton_proto.common import container_image__client_pb2 as _container_image__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor
JOB_ENVIRONMENT_REALTIME: JobEnvironment
JOB_ENVIRONMENT_RIFT_BATCH: JobEnvironment
JOB_ENVIRONMENT_RIFT_STREAM: JobEnvironment
JOB_ENVIRONMENT_UNSPECIFIED: JobEnvironment
REMOTE_COMPUTE_TYPE_CORE: RemoteComputeType
REMOTE_COMPUTE_TYPE_CUSTOM: RemoteComputeType
REMOTE_COMPUTE_TYPE_EXTENDED: RemoteComputeType
REMOTE_COMPUTE_TYPE_SNOWPARK_DEPRECATED_DO_NOT_USE: RemoteComputeType
REMOTE_ENVIRONMENT_STATUS_DELETING: RemoteEnvironmentStatus
REMOTE_ENVIRONMENT_STATUS_DELETION_FAILED: RemoteEnvironmentStatus
REMOTE_ENVIRONMENT_STATUS_ERROR: RemoteEnvironmentStatus
REMOTE_ENVIRONMENT_STATUS_PENDING: RemoteEnvironmentStatus
REMOTE_ENVIRONMENT_STATUS_READY: RemoteEnvironmentStatus

class DependentFeatureService(_message.Message):
    __slots__ = ["feature_service_name", "workspace_name"]
    FEATURE_SERVICE_NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_NAME_FIELD_NUMBER: ClassVar[int]
    feature_service_name: str
    workspace_name: str
    def __init__(self, workspace_name: Optional[str] = ..., feature_service_name: Optional[str] = ...) -> None: ...

class DependentFeatureView(_message.Message):
    __slots__ = ["feature_view_name", "workspace_name"]
    FEATURE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_NAME_FIELD_NUMBER: ClassVar[int]
    feature_view_name: str
    workspace_name: str
    def __init__(self, workspace_name: Optional[str] = ..., feature_view_name: Optional[str] = ...) -> None: ...

class ObjectStoreUploadPart(_message.Message):
    __slots__ = ["s3_upload_part"]
    S3_UPLOAD_PART_FIELD_NUMBER: ClassVar[int]
    s3_upload_part: S3UploadPart
    def __init__(self, s3_upload_part: Optional[Union[S3UploadPart, Mapping]] = ...) -> None: ...

class RealtimeEnvironment(_message.Message):
    __slots__ = ["feature_services", "image_info", "online_provisioned", "remote_function_uri", "tecton_transform_runtime_version"]
    FEATURE_SERVICES_FIELD_NUMBER: ClassVar[int]
    IMAGE_INFO_FIELD_NUMBER: ClassVar[int]
    ONLINE_PROVISIONED_FIELD_NUMBER: ClassVar[int]
    REMOTE_FUNCTION_URI_FIELD_NUMBER: ClassVar[int]
    TECTON_TRANSFORM_RUNTIME_VERSION_FIELD_NUMBER: ClassVar[int]
    feature_services: _containers.RepeatedCompositeFieldContainer[DependentFeatureService]
    image_info: _container_image__client_pb2.ContainerImage
    online_provisioned: bool
    remote_function_uri: str
    tecton_transform_runtime_version: str
    def __init__(self, tecton_transform_runtime_version: Optional[str] = ..., image_info: Optional[Union[_container_image__client_pb2.ContainerImage, Mapping]] = ..., remote_function_uri: Optional[str] = ..., feature_services: Optional[Iterable[Union[DependentFeatureService, Mapping]]] = ..., online_provisioned: bool = ...) -> None: ...

class RemoteComputeEnvironment(_message.Message):
    __slots__ = ["created_at", "created_by", "description", "feature_services", "id", "image_info", "name", "python_version", "realtime_job_environment", "requirements", "resolved_requirements", "rift_batch_job_environment", "s3_wheels_location", "sdk_version", "status", "status_details", "supported_job_environments", "type", "updated_at"]
    CREATED_AT_FIELD_NUMBER: ClassVar[int]
    CREATED_BY_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    FEATURE_SERVICES_FIELD_NUMBER: ClassVar[int]
    ID_FIELD_NUMBER: ClassVar[int]
    IMAGE_INFO_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    PYTHON_VERSION_FIELD_NUMBER: ClassVar[int]
    REALTIME_JOB_ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    REQUIREMENTS_FIELD_NUMBER: ClassVar[int]
    RESOLVED_REQUIREMENTS_FIELD_NUMBER: ClassVar[int]
    RIFT_BATCH_JOB_ENVIRONMENT_FIELD_NUMBER: ClassVar[int]
    S3_WHEELS_LOCATION_FIELD_NUMBER: ClassVar[int]
    SDK_VERSION_FIELD_NUMBER: ClassVar[int]
    STATUS_DETAILS_FIELD_NUMBER: ClassVar[int]
    STATUS_FIELD_NUMBER: ClassVar[int]
    SUPPORTED_JOB_ENVIRONMENTS_FIELD_NUMBER: ClassVar[int]
    TYPE_FIELD_NUMBER: ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: ClassVar[int]
    created_at: _timestamp_pb2.Timestamp
    created_by: _principal__client_pb2.Principal
    description: str
    feature_services: _containers.RepeatedCompositeFieldContainer[DependentFeatureService]
    id: str
    image_info: _container_image__client_pb2.ContainerImage
    name: str
    python_version: str
    realtime_job_environment: RealtimeEnvironment
    requirements: str
    resolved_requirements: str
    rift_batch_job_environment: RiftBatchEnvironment
    s3_wheels_location: str
    sdk_version: str
    status: RemoteEnvironmentStatus
    status_details: str
    supported_job_environments: _containers.RepeatedScalarFieldContainer[JobEnvironment]
    type: RemoteComputeType
    updated_at: _timestamp_pb2.Timestamp
    def __init__(self, id: Optional[str] = ..., name: Optional[str] = ..., type: Optional[Union[RemoteComputeType, str]] = ..., status: Optional[Union[RemoteEnvironmentStatus, str]] = ..., image_info: Optional[Union[_container_image__client_pb2.ContainerImage, Mapping]] = ..., created_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., updated_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., created_by: Optional[Union[_principal__client_pb2.Principal, Mapping]] = ..., description: Optional[str] = ..., python_version: Optional[str] = ..., requirements: Optional[str] = ..., resolved_requirements: Optional[str] = ..., s3_wheels_location: Optional[str] = ..., feature_services: Optional[Iterable[Union[DependentFeatureService, Mapping]]] = ..., realtime_job_environment: Optional[Union[RealtimeEnvironment, Mapping]] = ..., rift_batch_job_environment: Optional[Union[RiftBatchEnvironment, Mapping]] = ..., supported_job_environments: Optional[Iterable[Union[JobEnvironment, str]]] = ..., sdk_version: Optional[str] = ..., status_details: Optional[str] = ...) -> None: ...

class RemoteEnvironmentUploadInfo(_message.Message):
    __slots__ = ["environment_id", "s3_upload_info"]
    ENVIRONMENT_ID_FIELD_NUMBER: ClassVar[int]
    S3_UPLOAD_INFO_FIELD_NUMBER: ClassVar[int]
    environment_id: str
    s3_upload_info: S3UploadInfo
    def __init__(self, environment_id: Optional[str] = ..., s3_upload_info: Optional[Union[S3UploadInfo, Mapping]] = ...) -> None: ...

class RiftBatchEnvironment(_message.Message):
    __slots__ = ["cluster_environment_build_id", "image_info", "tecton_materialization_runtime_version"]
    CLUSTER_ENVIRONMENT_BUILD_ID_FIELD_NUMBER: ClassVar[int]
    IMAGE_INFO_FIELD_NUMBER: ClassVar[int]
    TECTON_MATERIALIZATION_RUNTIME_VERSION_FIELD_NUMBER: ClassVar[int]
    cluster_environment_build_id: str
    image_info: _container_image__client_pb2.ContainerImage
    tecton_materialization_runtime_version: str
    def __init__(self, tecton_materialization_runtime_version: Optional[str] = ..., image_info: Optional[Union[_container_image__client_pb2.ContainerImage, Mapping]] = ..., cluster_environment_build_id: Optional[str] = ...) -> None: ...

class S3UploadInfo(_message.Message):
    __slots__ = ["upload_id", "upload_parts"]
    UPLOAD_ID_FIELD_NUMBER: ClassVar[int]
    UPLOAD_PARTS_FIELD_NUMBER: ClassVar[int]
    upload_id: str
    upload_parts: _containers.RepeatedCompositeFieldContainer[S3UploadPart]
    def __init__(self, upload_id: Optional[str] = ..., upload_parts: Optional[Iterable[Union[S3UploadPart, Mapping]]] = ...) -> None: ...

class S3UploadPart(_message.Message):
    __slots__ = ["e_tag", "parent_upload_id", "part_number", "upload_url"]
    E_TAG_FIELD_NUMBER: ClassVar[int]
    PARENT_UPLOAD_ID_FIELD_NUMBER: ClassVar[int]
    PART_NUMBER_FIELD_NUMBER: ClassVar[int]
    UPLOAD_URL_FIELD_NUMBER: ClassVar[int]
    e_tag: str
    parent_upload_id: str
    part_number: int
    upload_url: str
    def __init__(self, parent_upload_id: Optional[str] = ..., part_number: Optional[int] = ..., e_tag: Optional[str] = ..., upload_url: Optional[str] = ...) -> None: ...

class JobEnvironment(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class RemoteEnvironmentStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class RemoteComputeType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
